from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.google_calendars_calendars_item_conference_properties import (
        GoogleCalendarsCalendarsItemConferenceProperties,
    )
    from ..models.google_calendars_calendars_item_default_reminders import GoogleCalendarsCalendarsItemDefaultReminders
    from ..models.google_calendars_calendars_item_notification_settings import (
        GoogleCalendarsCalendarsItemNotificationSettings,
    )


T = TypeVar("T", bound="GoogleCalendarsCalendarsItem")


@attr.s(auto_attribs=True)
class GoogleCalendarsCalendarsItem:
    """A google calendar

    Attributes:
        kind (Union[Unset, str]): The kind of the calendar
        etag (Union[Unset, str]): The etag of the calendar
        id (Union[Unset, str]): The id of the calendar
        summary (Union[Unset, str]): The summary of the calendar
        description (Union[Unset, str]): The description of the calendar
        location (Union[Unset, str]): The location of the calendar
        time_zone (Union[Unset, str]): The timeZone of the calendar
        summary_override (Union[Unset, str]): The summaryOverride of the calendar
        color_id (Union[Unset, str]): The colorId of the calendar
        background_color (Union[Unset, str]): The backgroundColor of the calendar
        foreground_color (Union[Unset, str]): The foregroundColor of the calendar
        hidden (Union[Unset, bool]): Whether the calendar is hidden
        selected (Union[Unset, bool]): Whether the calendar is selected
        access_role (Union[Unset, str]): The accessRole of the calendar
        default_reminders (Union[Unset, GoogleCalendarsCalendarsItemDefaultReminders]): The defaultReminders of the
            calendar
        notification_settings (Union[Unset, GoogleCalendarsCalendarsItemNotificationSettings]): The notificationSettings
            of the calendar
        primary (Union[Unset, bool]): Whether the calendar is primary
        deleted (Union[Unset, bool]): Whether the calendar is deleted
        conference_properties (Union[Unset, GoogleCalendarsCalendarsItemConferenceProperties]): The conferenceProperties
            of the calendar
    """

    kind: Union[Unset, str] = UNSET
    etag: Union[Unset, str] = UNSET
    id: Union[Unset, str] = UNSET
    summary: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    location: Union[Unset, str] = UNSET
    time_zone: Union[Unset, str] = UNSET
    summary_override: Union[Unset, str] = UNSET
    color_id: Union[Unset, str] = UNSET
    background_color: Union[Unset, str] = UNSET
    foreground_color: Union[Unset, str] = UNSET
    hidden: Union[Unset, bool] = UNSET
    selected: Union[Unset, bool] = UNSET
    access_role: Union[Unset, str] = UNSET
    default_reminders: Union[Unset, "GoogleCalendarsCalendarsItemDefaultReminders"] = UNSET
    notification_settings: Union[Unset, "GoogleCalendarsCalendarsItemNotificationSettings"] = UNSET
    primary: Union[Unset, bool] = UNSET
    deleted: Union[Unset, bool] = UNSET
    conference_properties: Union[Unset, "GoogleCalendarsCalendarsItemConferenceProperties"] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        kind = self.kind
        etag = self.etag
        id = self.id
        summary = self.summary
        description = self.description
        location = self.location
        time_zone = self.time_zone
        summary_override = self.summary_override
        color_id = self.color_id
        background_color = self.background_color
        foreground_color = self.foreground_color
        hidden = self.hidden
        selected = self.selected
        access_role = self.access_role
        default_reminders: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.default_reminders, Unset):
            default_reminders = self.default_reminders.to_dict()

        notification_settings: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.notification_settings, Unset):
            notification_settings = self.notification_settings.to_dict()

        primary = self.primary
        deleted = self.deleted
        conference_properties: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.conference_properties, Unset):
            conference_properties = self.conference_properties.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if kind is not UNSET:
            field_dict["kind"] = kind
        if etag is not UNSET:
            field_dict["etag"] = etag
        if id is not UNSET:
            field_dict["id"] = id
        if summary is not UNSET:
            field_dict["summary"] = summary
        if description is not UNSET:
            field_dict["description"] = description
        if location is not UNSET:
            field_dict["location"] = location
        if time_zone is not UNSET:
            field_dict["timeZone"] = time_zone
        if summary_override is not UNSET:
            field_dict["summaryOverride"] = summary_override
        if color_id is not UNSET:
            field_dict["colorId"] = color_id
        if background_color is not UNSET:
            field_dict["backgroundColor"] = background_color
        if foreground_color is not UNSET:
            field_dict["foregroundColor"] = foreground_color
        if hidden is not UNSET:
            field_dict["hidden"] = hidden
        if selected is not UNSET:
            field_dict["selected"] = selected
        if access_role is not UNSET:
            field_dict["accessRole"] = access_role
        if default_reminders is not UNSET:
            field_dict["defaultReminders"] = default_reminders
        if notification_settings is not UNSET:
            field_dict["notificationSettings"] = notification_settings
        if primary is not UNSET:
            field_dict["primary"] = primary
        if deleted is not UNSET:
            field_dict["deleted"] = deleted
        if conference_properties is not UNSET:
            field_dict["conferenceProperties"] = conference_properties

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.google_calendars_calendars_item_conference_properties import (
            GoogleCalendarsCalendarsItemConferenceProperties,
        )
        from ..models.google_calendars_calendars_item_default_reminders import (
            GoogleCalendarsCalendarsItemDefaultReminders,
        )
        from ..models.google_calendars_calendars_item_notification_settings import (
            GoogleCalendarsCalendarsItemNotificationSettings,
        )

        d = src_dict.copy()
        kind = d.pop("kind", UNSET) or UNSET

        etag = d.pop("etag", UNSET) or UNSET

        id = d.pop("id", UNSET) or UNSET

        summary = d.pop("summary", UNSET) or UNSET

        description = d.pop("description", UNSET) or UNSET

        location = d.pop("location", UNSET) or UNSET

        time_zone = d.pop("timeZone", UNSET) or UNSET

        summary_override = d.pop("summaryOverride", UNSET) or UNSET

        color_id = d.pop("colorId", UNSET) or UNSET

        background_color = d.pop("backgroundColor", UNSET) or UNSET

        foreground_color = d.pop("foregroundColor", UNSET) or UNSET

        hidden = d.pop("hidden", UNSET) or UNSET

        selected = d.pop("selected", UNSET) or UNSET

        access_role = d.pop("accessRole", UNSET) or UNSET

        _default_reminders = d.pop("defaultReminders", UNSET) or UNSET
        default_reminders: Union[Unset, GoogleCalendarsCalendarsItemDefaultReminders]
        if isinstance(_default_reminders, Unset):
            default_reminders = UNSET
        else:
            default_reminders = GoogleCalendarsCalendarsItemDefaultReminders.from_dict(_default_reminders)

        _notification_settings = d.pop("notificationSettings", UNSET) or UNSET
        notification_settings: Union[Unset, GoogleCalendarsCalendarsItemNotificationSettings]
        if isinstance(_notification_settings, Unset):
            notification_settings = UNSET
        else:
            notification_settings = GoogleCalendarsCalendarsItemNotificationSettings.from_dict(_notification_settings)

        primary = d.pop("primary", UNSET) or UNSET

        deleted = d.pop("deleted", UNSET) or UNSET

        _conference_properties = d.pop("conferenceProperties", UNSET) or UNSET
        conference_properties: Union[Unset, GoogleCalendarsCalendarsItemConferenceProperties]
        if isinstance(_conference_properties, Unset):
            conference_properties = UNSET
        else:
            conference_properties = GoogleCalendarsCalendarsItemConferenceProperties.from_dict(_conference_properties)

        google_calendars_calendars_item = cls(
            kind=kind,
            etag=etag,
            id=id,
            summary=summary,
            description=description,
            location=location,
            time_zone=time_zone,
            summary_override=summary_override,
            color_id=color_id,
            background_color=background_color,
            foreground_color=foreground_color,
            hidden=hidden,
            selected=selected,
            access_role=access_role,
            default_reminders=default_reminders,
            notification_settings=notification_settings,
            primary=primary,
            deleted=deleted,
            conference_properties=conference_properties,
        )

        google_calendars_calendars_item.additional_properties = d
        return google_calendars_calendars_item

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Create an authenticated client
import os
from functools import wraps
from time import sleep

from ..client import AuthenticatedClient

token = os.environ.get("BRUINEN_API_KEY")
base_url = os.environ.get("BRUINEN_API_URL")

client = AuthenticatedClient(
    base_url=base_url or "http://localhost:3000",
    token=token,
    prefix="",
    auth_header_name="X-API-Key",
)

from ..client.api.confirm import create as confirm_create
from ..client.api.confirm import find_one as confirm_get
from ..client.models import CreateConfirmDto


def confirm_action(
    action,
    delivery_channel,
    params={},
    delivery_address=None,
    on_reject=None,
    on_magic_link=None,
):
    """confirm an action before calling function"""

    def decorate(fn):
        @wraps(fn)
        def wrapper(*args, **kwargs):
            confirmation = confirm_create.sync(
                client=client,
                json_body=CreateConfirmDto.from_dict(
                    {
                        "type": "test",
                        "deliveryAddress": delivery_address,
                        "deliveryChannel": delivery_channel,
                        "action": action,
                        "params": params,
                    }
                ),
            )
            if delivery_channel == "magic-link":
                if on_magic_link:
                    on_magic_link(confirmation.to_dict())
            conf = None
            while True:
                print("Checking for confirmation")
                conf = confirm_get.sync(client=client, id=confirmation.confirmation_id)
                conf = conf.to_dict()
                print(conf)
                if "respondedAt" in conf:
                    break
                sleep(3)
            if conf["status"] == "denied":
                if on_reject:
                    on_reject()
                return
            if conf["status"] == "confirmed":
                fn(*args, **kwargs)
                return

        return wrapper

    return decorate



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.google_calendars_calendars_item_notification_settings_notifications_item import (
        GoogleCalendarsCalendarsItemNotificationSettingsNotificationsItem,
    )


T = TypeVar("T", bound="GoogleCalendarsCalendarsItemNotificationSettings")


@attr.s(auto_attribs=True)
class GoogleCalendarsCalendarsItemNotificationSettings:
    """The notificationSettings of the calendar

    Attributes:
        notifications (Union[Unset, List['GoogleCalendarsCalendarsItemNotificationSettingsNotificationsItem']]): The
            notifications of the notificationSettings
    """

    notifications: Union[Unset, List["GoogleCalendarsCalendarsItemNotificationSettingsNotificationsItem"]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        notifications: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.notifications, Unset):
            notifications = []
            for notifications_item_data in self.notifications:
                notifications_item = notifications_item_data.to_dict()

                notifications.append(notifications_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if notifications is not UNSET:
            field_dict["notifications"] = notifications

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.google_calendars_calendars_item_notification_settings_notifications_item import (
            GoogleCalendarsCalendarsItemNotificationSettingsNotificationsItem,
        )

        d = src_dict.copy()
        notifications = []
        _notifications = d.pop("notifications", UNSET) or UNSET
        for notifications_item_data in _notifications or []:
            notifications_item = GoogleCalendarsCalendarsItemNotificationSettingsNotificationsItem.from_dict(
                notifications_item_data
            )

            notifications.append(notifications_item)

        google_calendars_calendars_item_notification_settings = cls(
            notifications=notifications,
        )

        google_calendars_calendars_item_notification_settings.additional_properties = d
        return google_calendars_calendars_item_notification_settings

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# TODO figure out how the generator handles multiple methods (GET, POST, etc.) with the same name

import json
from langchain.base_language import BaseLanguageModel
from langchain.callbacks.manager import AsyncCallbackManagerForToolRun, CallbackManagerForToolRun
from langchain.output_parsers import PydanticOutputParser
from langchain.prompts.prompt import PromptTemplate
from langchain.tools import BaseTool
from pydantic import BaseModel, Field
from typing import Callable, List, Optional
from urllib.parse import quote

from ..client import AuthenticatedClient
from ..client.api.accounts import find_all_accounts_for_user
from ..client.api.auth import get_user_auth_token
from ..client.models import Auth, ReturnedAccountDto
from ..client.types import Response

from ..client.api.sources import google_controller_profile
from ..client.models import GoogleProfile
from ..client.api.sources import google_controller_drafts
from ..client.models import GoogleDrafts
from ..client.api.sources import google_controller_parsed_drafts
from ..client.models import GoogleParsedDrafts
from ..client.api.sources import google_controller_draft
from ..client.models import GoogleDraft
from ..client.api.sources import google_controller_parsed_draft
from ..client.models import GoogleParsedDraft
from ..client.api.sources import google_controller_labels
from ..client.models import GoogleLabels
from ..client.api.sources import google_controller_label
from ..client.models import GoogleLabel
from ..client.api.sources import google_controller_messages
from ..client.models import GoogleMessages
from ..client.api.sources import google_controller_parsed_messages
from ..client.models import GoogleParsedMessages
from ..client.api.sources import google_controller_message
from ..client.models import GoogleMessage
from ..client.api.sources import google_controller_parsed_message
from ..client.models import GoogleParsedMessage
from ..client.api.sources import google_controller_threads
from ..client.models import GoogleThreads
from ..client.api.sources import google_controller_parsed_threads
from ..client.models import GoogleParsedThreads
from ..client.api.sources import google_controller_thread
from ..client.models import GoogleThread
from ..client.api.sources import google_controller_parsed_thread
from ..client.models import GoogleParsedThread
from ..client.api.sources import google_controller_calendars
from ..client.models import GoogleCalendars
from ..client.api.sources import google_controller_calendar
from ..client.models import GoogleCalendar
from ..client.api.sources import google_controller_events
from ..client.models import GoogleEvents
from ..client.api.sources import google_controller_event
from ..client.models import GoogleEvent


class GoogleAuthenticatorTool(BaseTool):
    name = 'google_authenticator_tool'
    description = """Useful for when a user's Google account is not authenticated.

    Input to the tool should be an empty string.

    The response from the tool will be a URL that you return to the user for them to complete auth.
    The URL will be your final answer.
    """

    client: AuthenticatedClient
    user_id: str
    server: str = 'https://ui.bruinen.co'
    source_policy_id: str = None
    redirect_url: str

    def _run(
        self,
        query: str,
        run_manager: Optional[CallbackManagerForToolRun] = None
    ) -> str:
        """Run the tool."""
        response: Response[Auth] = get_user_auth_token.sync_detailed(client=self.client, user_id=self.user_id)
        user_token = response.parsed.access_token

        if self.source_policy_id is not None:
            source = [{ 'name': 'google', 'sourcePolicyId': self.source_policy_id }]
        else:
            source = [{ 'name': 'google' }]
        encoded_source = quote(json.dumps(source))
        
        return self.server + '/connect' + '?userToken=' + quote(user_token) + '&sources=' + encoded_source + '&defaultRedirectUrl=' + quote(self.redirect_url)

    # TODO implement async version
    async def _arun(
        self,
        query: str,
        run_manager: Optional[AsyncCallbackManagerForToolRun] = None,
    ) -> str:
        """Run the tool asynchronously."""
        return await self._run(query, run_manager)


class GoogleGetProfileTool(BaseTool):
    name = "google_get_profile_tool"
    description = """Useful for when you need to get a user's Google profile.
    
    Input should be the question that you want to know the answer to.
    
    Output will be the text response from the Google API.
    """

    client: AuthenticatedClient
    user_id: str
    parse_output: Optional[Callable[[GoogleProfile], str]] = None
    
    def _run(self, query: str, run_manager: Optional[CallbackManagerForToolRun] = None) -> str:
        """Run the tool."""
        
        response: Response[List["ReturnedAccountDto"]] = find_all_accounts_for_user.sync_detailed(
            client=self.client, user_id=self.user_id
        )
        if not 200 <= response.status_code < 300:
            return "Error pulling the user's Google account."
        accounts: List["ReturnedAccountDto"] = response.parsed

        account_id = ""
        for account in accounts:
            if account.source == "google":
                account_id = account.id
        if account_id == "":
            return "The user has not connected their Google account; you should try authenticating Google first."
        else:
            response: Response[GoogleProfile] = google_controller_profile.sync_detailed(
                client=self.client,
                account_id=account_id
            )
            if not 200 <= response.status_code < 300:
                return "Error when attempting to get the user's Google profile."

            if self.parse_output is None:
                return json.dumps(response.parsed.to_dict())
            else:
                return self.parse_output(response.parsed, query)

    # TODO implement async version
    async def _arun(
        self,
        query: str,
        run_manager: Optional[AsyncCallbackManagerForToolRun] = None,
    ) -> str:
        """Run the tool asynchronously."""
        return await self._run(query, run_manager)


class GoogleGetDraftsTool(BaseTool):
    name = "google_get_drafts_tool"
    description = """Useful for when you need to get a user's Google drafts.
    
    Input should be a string containing the question that you want to know the answer to.
    Do not pass parameters as JSON, instead pass the string to the tool as is.
    
    Output will be the text response from the Google API.
    """

    class GoogleGetDraftsToolInputSchema(BaseModel):
        q: Optional[str] = Field(description="The query for your drafts")
        pageToken: Optional[str] = Field(description="The page token for your drafts")
        
    input_schema = GoogleGetDraftsToolInputSchema

    client: AuthenticatedClient
    user_id: str
    llm: Optional[BaseLanguageModel]
    parse_parameters: Optional[Callable[[str], GoogleGetDraftsToolInputSchema]] = None
    parse_output: Optional[Callable[[GoogleDrafts], str]] = None
    
    def _parse_parameters(self, _query: str) -> GoogleGetDraftsToolInputSchema:
        """Parse the input passed to the tool when running."""
        parser = PydanticOutputParser(pydantic_object=self.GoogleGetDraftsToolInputSchema)
        prompt = PromptTemplate(
            template="""Parse the provided input string.
            For values that are not marked as required, if no value is provided, return a null value.
            {format_instructions}
            Here is the input:
            {query}""",
            input_variables=["query"],
            partial_variables={"format_instructions": parser.get_format_instructions()},
        )
        _input = prompt.format_prompt(query=_query)
        if self.llm is None:
            return "Error: No language model provided for input parsing."
        output = self.llm(_input.to_string())
        return parser.parse(output)
    
    def _run(self, query: str, run_manager: Optional[CallbackManagerForToolRun] = None) -> str:
        """Run the tool."""
        
        # If no custom input parameter parser is passed, use the default one
        if self.parse_parameters is None:
            parsed_parameters = self._parse_parameters(query)
        else: 
            parsed_parameters = self.parse_parameters(query)
        
        response: Response[List["ReturnedAccountDto"]] = find_all_accounts_for_user.sync_detailed(
            client=self.client, user_id=self.user_id
        )
        if not 200 <= response.status_code < 300:
            return "Error pulling the user's Google account."
        accounts: List["ReturnedAccountDto"] = response.parsed

        account_id = ""
        for account in accounts:
            if account.source == "google":
                account_id = account.id
        if account_id == "":
            return "The user has not connected their Google account; you should try authenticating Google first."
        else:
            response: Response[GoogleDrafts] = google_controller_drafts.sync_detailed(
                client=self.client,
                account_id=account_id,
                q=parsed_parameters.q,
                page_token=parsed_parameters.pageToken
            )
            if not 200 <= response.status_code < 300:
                return "Error when attempting to get the user's Google drafts."

            if self.parse_output is None:
                return json.dumps(response.parsed.to_dict())
            else:
                return self.parse_output(response.parsed, query)

    # TODO implement async version
    async def _arun(
        self,
        query: str,
        run_manager: Optional[AsyncCallbackManagerForToolRun] = None,
    ) -> str:
        """Run the tool asynchronously."""
        return await self._run(query, run_manager)


class GoogleGetParsedDraftsTool(BaseTool):
    name = "google_get_parsed_drafts_tool"
    description = """Useful for when you need to get a user's Google parsed_drafts.
    
    Input should be a string containing the question that you want to know the answer to.
    Do not pass parameters as JSON, instead pass the string to the tool as is.
    
    Output will be the text response from the Google API.
    """

    class GoogleGetParsedDraftsToolInputSchema(BaseModel):
        q: Optional[str] = Field(description="The query for your drafts")
        pageToken: Optional[str] = Field(description="The page token for your drafts")
        
    input_schema = GoogleGetParsedDraftsToolInputSchema

    client: AuthenticatedClient
    user_id: str
    llm: Optional[BaseLanguageModel]
    parse_parameters: Optional[Callable[[str], GoogleGetParsedDraftsToolInputSchema]] = None
    parse_output: Optional[Callable[[GoogleParsedDrafts], str]] = None
    
    def _parse_parameters(self, _query: str) -> GoogleGetParsedDraftsToolInputSchema:
        """Parse the input passed to the tool when running."""
        parser = PydanticOutputParser(pydantic_object=self.GoogleGetParsedDraftsToolInputSchema)
        prompt = PromptTemplate(
            template="""Parse the provided input string.
            For values that are not marked as required, if no value is provided, return a null value.
            {format_instructions}
            Here is the input:
            {query}""",
            input_variables=["query"],
            partial_variables={"format_instructions": parser.get_format_instructions()},
        )
        _input = prompt.format_prompt(query=_query)
        if self.llm is None:
            return "Error: No language model provided for input parsing."
        output = self.llm(_input.to_string())
        return parser.parse(output)
    
    def _run(self, query: str, run_manager: Optional[CallbackManagerForToolRun] = None) -> str:
        """Run the tool."""
        
        # If no custom input parameter parser is passed, use the default one
        if self.parse_parameters is None:
            parsed_parameters = self._parse_parameters(query)
        else: 
            parsed_parameters = self.parse_parameters(query)
        
        response: Response[List["ReturnedAccountDto"]] = find_all_accounts_for_user.sync_detailed(
            client=self.client, user_id=self.user_id
        )
        if not 200 <= response.status_code < 300:
            return "Error pulling the user's Google account."
        accounts: List["ReturnedAccountDto"] = response.parsed

        account_id = ""
        for account in accounts:
            if account.source == "google":
                account_id = account.id
        if account_id == "":
            return "The user has not connected their Google account; you should try authenticating Google first."
        else:
            response: Response[GoogleParsedDrafts] = google_controller_parsed_drafts.sync_detailed(
                client=self.client,
                account_id=account_id,
                q=parsed_parameters.q,
                page_token=parsed_parameters.pageToken
            )
            if not 200 <= response.status_code < 300:
                return "Error when attempting to get the user's Google parsed_drafts."

            if self.parse_output is None:
                return json.dumps(response.parsed.to_dict())
            else:
                return self.parse_output(response.parsed, query)

    # TODO implement async version
    async def _arun(
        self,
        query: str,
        run_manager: Optional[AsyncCallbackManagerForToolRun] = None,
    ) -> str:
        """Run the tool asynchronously."""
        return await self._run(query, run_manager)


class GoogleGetDraftTool(BaseTool):
    name = "google_get_draft_tool"
    description = """Useful for when you need to get a user's Google draft.
    
    Input should be a string containing the question that you want to know the answer to.
    Do not pass parameters as JSON, instead pass the string to the tool as is.
    
    Output will be the text response from the Google API.
    """

    class GoogleGetDraftToolInputSchema(BaseModel):
        draftId: str = Field(description="The id of the draft")
        
    input_schema = GoogleGetDraftToolInputSchema

    client: AuthenticatedClient
    user_id: str
    llm: Optional[BaseLanguageModel]
    parse_parameters: Optional[Callable[[str], GoogleGetDraftToolInputSchema]] = None
    parse_output: Optional[Callable[[GoogleDraft], str]] = None
    
    def _parse_parameters(self, _query: str) -> GoogleGetDraftToolInputSchema:
        """Parse the input passed to the tool when running."""
        parser = PydanticOutputParser(pydantic_object=self.GoogleGetDraftToolInputSchema)
        prompt = PromptTemplate(
            template="""Parse the provided input string.
            For values that are not marked as required, if no value is provided, return a null value.
            {format_instructions}
            Here is the input:
            {query}""",
            input_variables=["query"],
            partial_variables={"format_instructions": parser.get_format_instructions()},
        )
        _input = prompt.format_prompt(query=_query)
        if self.llm is None:
            return "Error: No language model provided for input parsing."
        output = self.llm(_input.to_string())
        return parser.parse(output)
    
    def _run(self, query: str, run_manager: Optional[CallbackManagerForToolRun] = None) -> str:
        """Run the tool."""
        
        # If no custom input parameter parser is passed, use the default one
        if self.parse_parameters is None:
            parsed_parameters = self._parse_parameters(query)
        else: 
            parsed_parameters = self.parse_parameters(query)
        
        response: Response[List["ReturnedAccountDto"]] = find_all_accounts_for_user.sync_detailed(
            client=self.client, user_id=self.user_id
        )
        if not 200 <= response.status_code < 300:
            return "Error pulling the user's Google account."
        accounts: List["ReturnedAccountDto"] = response.parsed

        account_id = ""
        for account in accounts:
            if account.source == "google":
                account_id = account.id
        if account_id == "":
            return "The user has not connected their Google account; you should try authenticating Google first."
        else:
            response: Response[GoogleDraft] = google_controller_draft.sync_detailed(
                client=self.client,
                account_id=account_id,
                draft_id=parsed_parameters.draftId
            )
            if not 200 <= response.status_code < 300:
                return "Error when attempting to get the user's Google draft."

            if self.parse_output is None:
                return json.dumps(response.parsed.to_dict())
            else:
                return self.parse_output(response.parsed, query)

    # TODO implement async version
    async def _arun(
        self,
        query: str,
        run_manager: Optional[AsyncCallbackManagerForToolRun] = None,
    ) -> str:
        """Run the tool asynchronously."""
        return await self._run(query, run_manager)


class GoogleGetParsedDraftTool(BaseTool):
    name = "google_get_parsed_draft_tool"
    description = """Useful for when you need to get a user's Google parsed_draft.
    
    Input should be a string containing the question that you want to know the answer to.
    Do not pass parameters as JSON, instead pass the string to the tool as is.
    
    Output will be the text response from the Google API.
    """

    class GoogleGetParsedDraftToolInputSchema(BaseModel):
        draftId: str = Field(description="The id of the draft")
        
    input_schema = GoogleGetParsedDraftToolInputSchema

    client: AuthenticatedClient
    user_id: str
    llm: Optional[BaseLanguageModel]
    parse_parameters: Optional[Callable[[str], GoogleGetParsedDraftToolInputSchema]] = None
    parse_output: Optional[Callable[[GoogleParsedDraft], str]] = None
    
    def _parse_parameters(self, _query: str) -> GoogleGetParsedDraftToolInputSchema:
        """Parse the input passed to the tool when running."""
        parser = PydanticOutputParser(pydantic_object=self.GoogleGetParsedDraftToolInputSchema)
        prompt = PromptTemplate(
            template="""Parse the provided input string.
            For values that are not marked as required, if no value is provided, return a null value.
            {format_instructions}
            Here is the input:
            {query}""",
            input_variables=["query"],
            partial_variables={"format_instructions": parser.get_format_instructions()},
        )
        _input = prompt.format_prompt(query=_query)
        if self.llm is None:
            return "Error: No language model provided for input parsing."
        output = self.llm(_input.to_string())
        return parser.parse(output)
    
    def _run(self, query: str, run_manager: Optional[CallbackManagerForToolRun] = None) -> str:
        """Run the tool."""
        
        # If no custom input parameter parser is passed, use the default one
        if self.parse_parameters is None:
            parsed_parameters = self._parse_parameters(query)
        else: 
            parsed_parameters = self.parse_parameters(query)
        
        response: Response[List["ReturnedAccountDto"]] = find_all_accounts_for_user.sync_detailed(
            client=self.client, user_id=self.user_id
        )
        if not 200 <= response.status_code < 300:
            return "Error pulling the user's Google account."
        accounts: List["ReturnedAccountDto"] = response.parsed

        account_id = ""
        for account in accounts:
            if account.source == "google":
                account_id = account.id
        if account_id == "":
            return "The user has not connected their Google account; you should try authenticating Google first."
        else:
            response: Response[GoogleParsedDraft] = google_controller_parsed_draft.sync_detailed(
                client=self.client,
                account_id=account_id,
                draft_id=parsed_parameters.draftId
            )
            if not 200 <= response.status_code < 300:
                return "Error when attempting to get the user's Google parsed_draft."

            if self.parse_output is None:
                return json.dumps(response.parsed.to_dict())
            else:
                return self.parse_output(response.parsed, query)

    # TODO implement async version
    async def _arun(
        self,
        query: str,
        run_manager: Optional[AsyncCallbackManagerForToolRun] = None,
    ) -> str:
        """Run the tool asynchronously."""
        return await self._run(query, run_manager)


class GoogleGetLabelsTool(BaseTool):
    name = "google_get_labels_tool"
    description = """Useful for when you need to get a user's Google labels.
    
    Input should be the question that you want to know the answer to.
    
    Output will be the text response from the Google API.
    """

    client: AuthenticatedClient
    user_id: str
    parse_output: Optional[Callable[[GoogleLabels], str]] = None
    
    def _run(self, query: str, run_manager: Optional[CallbackManagerForToolRun] = None) -> str:
        """Run the tool."""
        
        response: Response[List["ReturnedAccountDto"]] = find_all_accounts_for_user.sync_detailed(
            client=self.client, user_id=self.user_id
        )
        if not 200 <= response.status_code < 300:
            return "Error pulling the user's Google account."
        accounts: List["ReturnedAccountDto"] = response.parsed

        account_id = ""
        for account in accounts:
            if account.source == "google":
                account_id = account.id
        if account_id == "":
            return "The user has not connected their Google account; you should try authenticating Google first."
        else:
            response: Response[GoogleLabels] = google_controller_labels.sync_detailed(
                client=self.client,
                account_id=account_id
            )
            if not 200 <= response.status_code < 300:
                return "Error when attempting to get the user's Google labels."

            if self.parse_output is None:
                return json.dumps(response.parsed.to_dict())
            else:
                return self.parse_output(response.parsed, query)

    # TODO implement async version
    async def _arun(
        self,
        query: str,
        run_manager: Optional[AsyncCallbackManagerForToolRun] = None,
    ) -> str:
        """Run the tool asynchronously."""
        return await self._run(query, run_manager)


class GoogleGetLabelTool(BaseTool):
    name = "google_get_label_tool"
    description = """Useful for when you need to get a user's Google label.
    
    Input should be a string containing the question that you want to know the answer to.
    Do not pass parameters as JSON, instead pass the string to the tool as is.
    
    Output will be the text response from the Google API.
    """

    class GoogleGetLabelToolInputSchema(BaseModel):
        labelId: str = Field(description="The id of the label")
        
    input_schema = GoogleGetLabelToolInputSchema

    client: AuthenticatedClient
    user_id: str
    llm: Optional[BaseLanguageModel]
    parse_parameters: Optional[Callable[[str], GoogleGetLabelToolInputSchema]] = None
    parse_output: Optional[Callable[[GoogleLabel], str]] = None
    
    def _parse_parameters(self, _query: str) -> GoogleGetLabelToolInputSchema:
        """Parse the input passed to the tool when running."""
        parser = PydanticOutputParser(pydantic_object=self.GoogleGetLabelToolInputSchema)
        prompt = PromptTemplate(
            template="""Parse the provided input string.
            For values that are not marked as required, if no value is provided, return a null value.
            {format_instructions}
            Here is the input:
            {query}""",
            input_variables=["query"],
            partial_variables={"format_instructions": parser.get_format_instructions()},
        )
        _input = prompt.format_prompt(query=_query)
        if self.llm is None:
            return "Error: No language model provided for input parsing."
        output = self.llm(_input.to_string())
        return parser.parse(output)
    
    def _run(self, query: str, run_manager: Optional[CallbackManagerForToolRun] = None) -> str:
        """Run the tool."""
        
        # If no custom input parameter parser is passed, use the default one
        if self.parse_parameters is None:
            parsed_parameters = self._parse_parameters(query)
        else: 
            parsed_parameters = self.parse_parameters(query)
        
        response: Response[List["ReturnedAccountDto"]] = find_all_accounts_for_user.sync_detailed(
            client=self.client, user_id=self.user_id
        )
        if not 200 <= response.status_code < 300:
            return "Error pulling the user's Google account."
        accounts: List["ReturnedAccountDto"] = response.parsed

        account_id = ""
        for account in accounts:
            if account.source == "google":
                account_id = account.id
        if account_id == "":
            return "The user has not connected their Google account; you should try authenticating Google first."
        else:
            response: Response[GoogleLabel] = google_controller_label.sync_detailed(
                client=self.client,
                account_id=account_id,
                label_id=parsed_parameters.labelId
            )
            if not 200 <= response.status_code < 300:
                return "Error when attempting to get the user's Google label."

            if self.parse_output is None:
                return json.dumps(response.parsed.to_dict())
            else:
                return self.parse_output(response.parsed, query)

    # TODO implement async version
    async def _arun(
        self,
        query: str,
        run_manager: Optional[AsyncCallbackManagerForToolRun] = None,
    ) -> str:
        """Run the tool asynchronously."""
        return await self._run(query, run_manager)


class GoogleGetMessagesTool(BaseTool):
    name = "google_get_messages_tool"
    description = """Useful for when you need to get a user's Google messages.
    
    Input should be a string containing the question that you want to know the answer to.
    Do not pass parameters as JSON, instead pass the string to the tool as is.
    
    Output will be the text response from the Google API.
    """

    class GoogleGetMessagesToolInputSchema(BaseModel):
        q: Optional[str] = Field(description="The query of the messages")
        pageToken: Optional[str] = Field(description="The pageToken of the messages")
        labelIds: Optional[str] = Field(description="The labelIds of the messages")
        
    input_schema = GoogleGetMessagesToolInputSchema

    client: AuthenticatedClient
    user_id: str
    llm: Optional[BaseLanguageModel]
    parse_parameters: Optional[Callable[[str], GoogleGetMessagesToolInputSchema]] = None
    parse_output: Optional[Callable[[GoogleMessages], str]] = None
    
    def _parse_parameters(self, _query: str) -> GoogleGetMessagesToolInputSchema:
        """Parse the input passed to the tool when running."""
        parser = PydanticOutputParser(pydantic_object=self.GoogleGetMessagesToolInputSchema)
        prompt = PromptTemplate(
            template="""Parse the provided input string.
            For values that are not marked as required, if no value is provided, return a null value.
            {format_instructions}
            Here is the input:
            {query}""",
            input_variables=["query"],
            partial_variables={"format_instructions": parser.get_format_instructions()},
        )
        _input = prompt.format_prompt(query=_query)
        if self.llm is None:
            return "Error: No language model provided for input parsing."
        output = self.llm(_input.to_string())
        return parser.parse(output)
    
    def _run(self, query: str, run_manager: Optional[CallbackManagerForToolRun] = None) -> str:
        """Run the tool."""
        
        # If no custom input parameter parser is passed, use the default one
        if self.parse_parameters is None:
            parsed_parameters = self._parse_parameters(query)
        else: 
            parsed_parameters = self.parse_parameters(query)
        
        response: Response[List["ReturnedAccountDto"]] = find_all_accounts_for_user.sync_detailed(
            client=self.client, user_id=self.user_id
        )
        if not 200 <= response.status_code < 300:
            return "Error pulling the user's Google account."
        accounts: List["ReturnedAccountDto"] = response.parsed

        account_id = ""
        for account in accounts:
            if account.source == "google":
                account_id = account.id
        if account_id == "":
            return "The user has not connected their Google account; you should try authenticating Google first."
        else:
            response: Response[GoogleMessages] = google_controller_messages.sync_detailed(
                client=self.client,
                account_id=account_id,
                q=parsed_parameters.q,
                page_token=parsed_parameters.pageToken,
                label_ids=parsed_parameters.labelIds
            )
            if not 200 <= response.status_code < 300:
                return "Error when attempting to get the user's Google messages."

            if self.parse_output is None:
                return json.dumps(response.parsed.to_dict())
            else:
                return self.parse_output(response.parsed, query)

    # TODO implement async version
    async def _arun(
        self,
        query: str,
        run_manager: Optional[AsyncCallbackManagerForToolRun] = None,
    ) -> str:
        """Run the tool asynchronously."""
        return await self._run(query, run_manager)


class GoogleGetParsedMessagesTool(BaseTool):
    name = "google_get_parsed_messages_tool"
    description = """Useful for when you need to get a user's Google parsed_messages.
    
    Input should be a string containing the question that you want to know the answer to.
    Do not pass parameters as JSON, instead pass the string to the tool as is.
    
    Output will be the text response from the Google API.
    """

    class GoogleGetParsedMessagesToolInputSchema(BaseModel):
        q: Optional[str] = Field(description="The query of the messages")
        pageToken: Optional[str] = Field(description="The pageToken of the messages")
        labelIds: Optional[str] = Field(description="The labelIds of the messages")
        
    input_schema = GoogleGetParsedMessagesToolInputSchema

    client: AuthenticatedClient
    user_id: str
    llm: Optional[BaseLanguageModel]
    parse_parameters: Optional[Callable[[str], GoogleGetParsedMessagesToolInputSchema]] = None
    parse_output: Optional[Callable[[GoogleParsedMessages], str]] = None
    
    def _parse_parameters(self, _query: str) -> GoogleGetParsedMessagesToolInputSchema:
        """Parse the input passed to the tool when running."""
        parser = PydanticOutputParser(pydantic_object=self.GoogleGetParsedMessagesToolInputSchema)
        prompt = PromptTemplate(
            template="""Parse the provided input string.
            For values that are not marked as required, if no value is provided, return a null value.
            {format_instructions}
            Here is the input:
            {query}""",
            input_variables=["query"],
            partial_variables={"format_instructions": parser.get_format_instructions()},
        )
        _input = prompt.format_prompt(query=_query)
        if self.llm is None:
            return "Error: No language model provided for input parsing."
        output = self.llm(_input.to_string())
        return parser.parse(output)
    
    def _run(self, query: str, run_manager: Optional[CallbackManagerForToolRun] = None) -> str:
        """Run the tool."""
        
        # If no custom input parameter parser is passed, use the default one
        if self.parse_parameters is None:
            parsed_parameters = self._parse_parameters(query)
        else: 
            parsed_parameters = self.parse_parameters(query)
        
        response: Response[List["ReturnedAccountDto"]] = find_all_accounts_for_user.sync_detailed(
            client=self.client, user_id=self.user_id
        )
        if not 200 <= response.status_code < 300:
            return "Error pulling the user's Google account."
        accounts: List["ReturnedAccountDto"] = response.parsed

        account_id = ""
        for account in accounts:
            if account.source == "google":
                account_id = account.id
        if account_id == "":
            return "The user has not connected their Google account; you should try authenticating Google first."
        else:
            response: Response[GoogleParsedMessages] = google_controller_parsed_messages.sync_detailed(
                client=self.client,
                account_id=account_id,
                q=parsed_parameters.q,
                page_token=parsed_parameters.pageToken,
                label_ids=parsed_parameters.labelIds
            )
            if not 200 <= response.status_code < 300:
                return "Error when attempting to get the user's Google parsed_messages."

            if self.parse_output is None:
                return json.dumps(response.parsed.to_dict())
            else:
                return self.parse_output(response.parsed, query)

    # TODO implement async version
    async def _arun(
        self,
        query: str,
        run_manager: Optional[AsyncCallbackManagerForToolRun] = None,
    ) -> str:
        """Run the tool asynchronously."""
        return await self._run(query, run_manager)


class GoogleGetMessageTool(BaseTool):
    name = "google_get_message_tool"
    description = """Useful for when you need to get a user's Google message.
    
    Input should be a string containing the question that you want to know the answer to.
    Do not pass parameters as JSON, instead pass the string to the tool as is.
    
    Output will be the text response from the Google API.
    """

    class GoogleGetMessageToolInputSchema(BaseModel):
        messageId: str = Field(description="The id of the message")
        
    input_schema = GoogleGetMessageToolInputSchema

    client: AuthenticatedClient
    user_id: str
    llm: Optional[BaseLanguageModel]
    parse_parameters: Optional[Callable[[str], GoogleGetMessageToolInputSchema]] = None
    parse_output: Optional[Callable[[GoogleMessage], str]] = None
    
    def _parse_parameters(self, _query: str) -> GoogleGetMessageToolInputSchema:
        """Parse the input passed to the tool when running."""
        parser = PydanticOutputParser(pydantic_object=self.GoogleGetMessageToolInputSchema)
        prompt = PromptTemplate(
            template="""Parse the provided input string.
            For values that are not marked as required, if no value is provided, return a null value.
            {format_instructions}
            Here is the input:
            {query}""",
            input_variables=["query"],
            partial_variables={"format_instructions": parser.get_format_instructions()},
        )
        _input = prompt.format_prompt(query=_query)
        if self.llm is None:
            return "Error: No language model provided for input parsing."
        output = self.llm(_input.to_string())
        return parser.parse(output)
    
    def _run(self, query: str, run_manager: Optional[CallbackManagerForToolRun] = None) -> str:
        """Run the tool."""
        
        # If no custom input parameter parser is passed, use the default one
        if self.parse_parameters is None:
            parsed_parameters = self._parse_parameters(query)
        else: 
            parsed_parameters = self.parse_parameters(query)
        
        response: Response[List["ReturnedAccountDto"]] = find_all_accounts_for_user.sync_detailed(
            client=self.client, user_id=self.user_id
        )
        if not 200 <= response.status_code < 300:
            return "Error pulling the user's Google account."
        accounts: List["ReturnedAccountDto"] = response.parsed

        account_id = ""
        for account in accounts:
            if account.source == "google":
                account_id = account.id
        if account_id == "":
            return "The user has not connected their Google account; you should try authenticating Google first."
        else:
            response: Response[GoogleMessage] = google_controller_message.sync_detailed(
                client=self.client,
                account_id=account_id,
                message_id=parsed_parameters.messageId
            )
            if not 200 <= response.status_code < 300:
                return "Error when attempting to get the user's Google message."

            if self.parse_output is None:
                return json.dumps(response.parsed.to_dict())
            else:
                return self.parse_output(response.parsed, query)

    # TODO implement async version
    async def _arun(
        self,
        query: str,
        run_manager: Optional[AsyncCallbackManagerForToolRun] = None,
    ) -> str:
        """Run the tool asynchronously."""
        return await self._run(query, run_manager)


class GoogleGetParsedMessageTool(BaseTool):
    name = "google_get_parsed_message_tool"
    description = """Useful for when you need to get a user's Google parsed_message.
    
    Input should be a string containing the question that you want to know the answer to.
    Do not pass parameters as JSON, instead pass the string to the tool as is.
    
    Output will be the text response from the Google API.
    """

    class GoogleGetParsedMessageToolInputSchema(BaseModel):
        messageId: str = Field(description="The id of the message")
        
    input_schema = GoogleGetParsedMessageToolInputSchema

    client: AuthenticatedClient
    user_id: str
    llm: Optional[BaseLanguageModel]
    parse_parameters: Optional[Callable[[str], GoogleGetParsedMessageToolInputSchema]] = None
    parse_output: Optional[Callable[[GoogleParsedMessage], str]] = None
    
    def _parse_parameters(self, _query: str) -> GoogleGetParsedMessageToolInputSchema:
        """Parse the input passed to the tool when running."""
        parser = PydanticOutputParser(pydantic_object=self.GoogleGetParsedMessageToolInputSchema)
        prompt = PromptTemplate(
            template="""Parse the provided input string.
            For values that are not marked as required, if no value is provided, return a null value.
            {format_instructions}
            Here is the input:
            {query}""",
            input_variables=["query"],
            partial_variables={"format_instructions": parser.get_format_instructions()},
        )
        _input = prompt.format_prompt(query=_query)
        if self.llm is None:
            return "Error: No language model provided for input parsing."
        output = self.llm(_input.to_string())
        return parser.parse(output)
    
    def _run(self, query: str, run_manager: Optional[CallbackManagerForToolRun] = None) -> str:
        """Run the tool."""
        
        # If no custom input parameter parser is passed, use the default one
        if self.parse_parameters is None:
            parsed_parameters = self._parse_parameters(query)
        else: 
            parsed_parameters = self.parse_parameters(query)
        
        response: Response[List["ReturnedAccountDto"]] = find_all_accounts_for_user.sync_detailed(
            client=self.client, user_id=self.user_id
        )
        if not 200 <= response.status_code < 300:
            return "Error pulling the user's Google account."
        accounts: List["ReturnedAccountDto"] = response.parsed

        account_id = ""
        for account in accounts:
            if account.source == "google":
                account_id = account.id
        if account_id == "":
            return "The user has not connected their Google account; you should try authenticating Google first."
        else:
            response: Response[GoogleParsedMessage] = google_controller_parsed_message.sync_detailed(
                client=self.client,
                account_id=account_id,
                message_id=parsed_parameters.messageId
            )
            if not 200 <= response.status_code < 300:
                return "Error when attempting to get the user's Google parsed_message."

            if self.parse_output is None:
                return json.dumps(response.parsed.to_dict())
            else:
                return self.parse_output(response.parsed, query)

    # TODO implement async version
    async def _arun(
        self,
        query: str,
        run_manager: Optional[AsyncCallbackManagerForToolRun] = None,
    ) -> str:
        """Run the tool asynchronously."""
        return await self._run(query, run_manager)


class GoogleGetThreadsTool(BaseTool):
    name = "google_get_threads_tool"
    description = """Useful for when you need to get a user's Google threads.
    
    Input should be a string containing the question that you want to know the answer to.
    Do not pass parameters as JSON, instead pass the string to the tool as is.
    
    Output will be the text response from the Google API.
    """

    class GoogleGetThreadsToolInputSchema(BaseModel):
        q: Optional[str] = Field(description="The query of the threads")
        pageToken: Optional[str] = Field(description="The pageToken of the threads")
        labelIds: Optional[str] = Field(description="The labelIds of the threads")
        
    input_schema = GoogleGetThreadsToolInputSchema

    client: AuthenticatedClient
    user_id: str
    llm: Optional[BaseLanguageModel]
    parse_parameters: Optional[Callable[[str], GoogleGetThreadsToolInputSchema]] = None
    parse_output: Optional[Callable[[GoogleThreads], str]] = None
    
    def _parse_parameters(self, _query: str) -> GoogleGetThreadsToolInputSchema:
        """Parse the input passed to the tool when running."""
        parser = PydanticOutputParser(pydantic_object=self.GoogleGetThreadsToolInputSchema)
        prompt = PromptTemplate(
            template="""Parse the provided input string.
            For values that are not marked as required, if no value is provided, return a null value.
            {format_instructions}
            Here is the input:
            {query}""",
            input_variables=["query"],
            partial_variables={"format_instructions": parser.get_format_instructions()},
        )
        _input = prompt.format_prompt(query=_query)
        if self.llm is None:
            return "Error: No language model provided for input parsing."
        output = self.llm(_input.to_string())
        return parser.parse(output)
    
    def _run(self, query: str, run_manager: Optional[CallbackManagerForToolRun] = None) -> str:
        """Run the tool."""
        
        # If no custom input parameter parser is passed, use the default one
        if self.parse_parameters is None:
            parsed_parameters = self._parse_parameters(query)
        else: 
            parsed_parameters = self.parse_parameters(query)
        
        response: Response[List["ReturnedAccountDto"]] = find_all_accounts_for_user.sync_detailed(
            client=self.client, user_id=self.user_id
        )
        if not 200 <= response.status_code < 300:
            return "Error pulling the user's Google account."
        accounts: List["ReturnedAccountDto"] = response.parsed

        account_id = ""
        for account in accounts:
            if account.source == "google":
                account_id = account.id
        if account_id == "":
            return "The user has not connected their Google account; you should try authenticating Google first."
        else:
            response: Response[GoogleThreads] = google_controller_threads.sync_detailed(
                client=self.client,
                account_id=account_id,
                q=parsed_parameters.q,
                page_token=parsed_parameters.pageToken,
                label_ids=parsed_parameters.labelIds
            )
            if not 200 <= response.status_code < 300:
                return "Error when attempting to get the user's Google threads."

            if self.parse_output is None:
                return json.dumps(response.parsed.to_dict())
            else:
                return self.parse_output(response.parsed, query)

    # TODO implement async version
    async def _arun(
        self,
        query: str,
        run_manager: Optional[AsyncCallbackManagerForToolRun] = None,
    ) -> str:
        """Run the tool asynchronously."""
        return await self._run(query, run_manager)


class GoogleGetParsedThreadsTool(BaseTool):
    name = "google_get_parsed_threads_tool"
    description = """Useful for when you need to get a user's Google parsed_threads.
    
    Input should be a string containing the question that you want to know the answer to.
    Do not pass parameters as JSON, instead pass the string to the tool as is.
    
    Output will be the text response from the Google API.
    """

    class GoogleGetParsedThreadsToolInputSchema(BaseModel):
        q: Optional[str] = Field(description="The query of the threads")
        pageToken: Optional[str] = Field(description="The pageToken of the threads")
        labelIds: Optional[str] = Field(description="The labelIds of the threads")
        
    input_schema = GoogleGetParsedThreadsToolInputSchema

    client: AuthenticatedClient
    user_id: str
    llm: Optional[BaseLanguageModel]
    parse_parameters: Optional[Callable[[str], GoogleGetParsedThreadsToolInputSchema]] = None
    parse_output: Optional[Callable[[GoogleParsedThreads], str]] = None
    
    def _parse_parameters(self, _query: str) -> GoogleGetParsedThreadsToolInputSchema:
        """Parse the input passed to the tool when running."""
        parser = PydanticOutputParser(pydantic_object=self.GoogleGetParsedThreadsToolInputSchema)
        prompt = PromptTemplate(
            template="""Parse the provided input string.
            For values that are not marked as required, if no value is provided, return a null value.
            {format_instructions}
            Here is the input:
            {query}""",
            input_variables=["query"],
            partial_variables={"format_instructions": parser.get_format_instructions()},
        )
        _input = prompt.format_prompt(query=_query)
        if self.llm is None:
            return "Error: No language model provided for input parsing."
        output = self.llm(_input.to_string())
        return parser.parse(output)
    
    def _run(self, query: str, run_manager: Optional[CallbackManagerForToolRun] = None) -> str:
        """Run the tool."""
        
        # If no custom input parameter parser is passed, use the default one
        if self.parse_parameters is None:
            parsed_parameters = self._parse_parameters(query)
        else: 
            parsed_parameters = self.parse_parameters(query)
        
        response: Response[List["ReturnedAccountDto"]] = find_all_accounts_for_user.sync_detailed(
            client=self.client, user_id=self.user_id
        )
        if not 200 <= response.status_code < 300:
            return "Error pulling the user's Google account."
        accounts: List["ReturnedAccountDto"] = response.parsed

        account_id = ""
        for account in accounts:
            if account.source == "google":
                account_id = account.id
        if account_id == "":
            return "The user has not connected their Google account; you should try authenticating Google first."
        else:
            response: Response[GoogleParsedThreads] = google_controller_parsed_threads.sync_detailed(
                client=self.client,
                account_id=account_id,
                q=parsed_parameters.q,
                page_token=parsed_parameters.pageToken,
                label_ids=parsed_parameters.labelIds
            )
            if not 200 <= response.status_code < 300:
                return "Error when attempting to get the user's Google parsed_threads."

            if self.parse_output is None:
                return json.dumps(response.parsed.to_dict())
            else:
                return self.parse_output(response.parsed, query)

    # TODO implement async version
    async def _arun(
        self,
        query: str,
        run_manager: Optional[AsyncCallbackManagerForToolRun] = None,
    ) -> str:
        """Run the tool asynchronously."""
        return await self._run(query, run_manager)


class GoogleGetThreadTool(BaseTool):
    name = "google_get_thread_tool"
    description = """Useful for when you need to get a user's Google thread.
    
    Input should be a string containing the question that you want to know the answer to.
    Do not pass parameters as JSON, instead pass the string to the tool as is.
    
    Output will be the text response from the Google API.
    """

    class GoogleGetThreadToolInputSchema(BaseModel):
        threadId: str = Field(description="The id of the thread")
        
    input_schema = GoogleGetThreadToolInputSchema

    client: AuthenticatedClient
    user_id: str
    llm: Optional[BaseLanguageModel]
    parse_parameters: Optional[Callable[[str], GoogleGetThreadToolInputSchema]] = None
    parse_output: Optional[Callable[[GoogleThread], str]] = None
    
    def _parse_parameters(self, _query: str) -> GoogleGetThreadToolInputSchema:
        """Parse the input passed to the tool when running."""
        parser = PydanticOutputParser(pydantic_object=self.GoogleGetThreadToolInputSchema)
        prompt = PromptTemplate(
            template="""Parse the provided input string.
            For values that are not marked as required, if no value is provided, return a null value.
            {format_instructions}
            Here is the input:
            {query}""",
            input_variables=["query"],
            partial_variables={"format_instructions": parser.get_format_instructions()},
        )
        _input = prompt.format_prompt(query=_query)
        if self.llm is None:
            return "Error: No language model provided for input parsing."
        output = self.llm(_input.to_string())
        return parser.parse(output)
    
    def _run(self, query: str, run_manager: Optional[CallbackManagerForToolRun] = None) -> str:
        """Run the tool."""
        
        # If no custom input parameter parser is passed, use the default one
        if self.parse_parameters is None:
            parsed_parameters = self._parse_parameters(query)
        else: 
            parsed_parameters = self.parse_parameters(query)
        
        response: Response[List["ReturnedAccountDto"]] = find_all_accounts_for_user.sync_detailed(
            client=self.client, user_id=self.user_id
        )
        if not 200 <= response.status_code < 300:
            return "Error pulling the user's Google account."
        accounts: List["ReturnedAccountDto"] = response.parsed

        account_id = ""
        for account in accounts:
            if account.source == "google":
                account_id = account.id
        if account_id == "":
            return "The user has not connected their Google account; you should try authenticating Google first."
        else:
            response: Response[GoogleThread] = google_controller_thread.sync_detailed(
                client=self.client,
                account_id=account_id,
                thread_id=parsed_parameters.threadId
            )
            if not 200 <= response.status_code < 300:
                return "Error when attempting to get the user's Google thread."

            if self.parse_output is None:
                return json.dumps(response.parsed.to_dict())
            else:
                return self.parse_output(response.parsed, query)

    # TODO implement async version
    async def _arun(
        self,
        query: str,
        run_manager: Optional[AsyncCallbackManagerForToolRun] = None,
    ) -> str:
        """Run the tool asynchronously."""
        return await self._run(query, run_manager)


class GoogleGetParsedThreadTool(BaseTool):
    name = "google_get_parsed_thread_tool"
    description = """Useful for when you need to get a user's Google parsed_thread.
    
    Input should be a string containing the question that you want to know the answer to.
    Do not pass parameters as JSON, instead pass the string to the tool as is.
    
    Output will be the text response from the Google API.
    """

    class GoogleGetParsedThreadToolInputSchema(BaseModel):
        threadId: str = Field(description="The id of the thread")
        
    input_schema = GoogleGetParsedThreadToolInputSchema

    client: AuthenticatedClient
    user_id: str
    llm: Optional[BaseLanguageModel]
    parse_parameters: Optional[Callable[[str], GoogleGetParsedThreadToolInputSchema]] = None
    parse_output: Optional[Callable[[GoogleParsedThread], str]] = None
    
    def _parse_parameters(self, _query: str) -> GoogleGetParsedThreadToolInputSchema:
        """Parse the input passed to the tool when running."""
        parser = PydanticOutputParser(pydantic_object=self.GoogleGetParsedThreadToolInputSchema)
        prompt = PromptTemplate(
            template="""Parse the provided input string.
            For values that are not marked as required, if no value is provided, return a null value.
            {format_instructions}
            Here is the input:
            {query}""",
            input_variables=["query"],
            partial_variables={"format_instructions": parser.get_format_instructions()},
        )
        _input = prompt.format_prompt(query=_query)
        if self.llm is None:
            return "Error: No language model provided for input parsing."
        output = self.llm(_input.to_string())
        return parser.parse(output)
    
    def _run(self, query: str, run_manager: Optional[CallbackManagerForToolRun] = None) -> str:
        """Run the tool."""
        
        # If no custom input parameter parser is passed, use the default one
        if self.parse_parameters is None:
            parsed_parameters = self._parse_parameters(query)
        else: 
            parsed_parameters = self.parse_parameters(query)
        
        response: Response[List["ReturnedAccountDto"]] = find_all_accounts_for_user.sync_detailed(
            client=self.client, user_id=self.user_id
        )
        if not 200 <= response.status_code < 300:
            return "Error pulling the user's Google account."
        accounts: List["ReturnedAccountDto"] = response.parsed

        account_id = ""
        for account in accounts:
            if account.source == "google":
                account_id = account.id
        if account_id == "":
            return "The user has not connected their Google account; you should try authenticating Google first."
        else:
            response: Response[GoogleParsedThread] = google_controller_parsed_thread.sync_detailed(
                client=self.client,
                account_id=account_id,
                thread_id=parsed_parameters.threadId
            )
            if not 200 <= response.status_code < 300:
                return "Error when attempting to get the user's Google parsed_thread."

            if self.parse_output is None:
                return json.dumps(response.parsed.to_dict())
            else:
                return self.parse_output(response.parsed, query)

    # TODO implement async version
    async def _arun(
        self,
        query: str,
        run_manager: Optional[AsyncCallbackManagerForToolRun] = None,
    ) -> str:
        """Run the tool asynchronously."""
        return await self._run(query, run_manager)


class GoogleGetCalendarsTool(BaseTool):
    name = "google_get_calendars_tool"
    description = """Useful for when you need to get a user's Google calendars.
    
    Input should be a string containing the question that you want to know the answer to.
    Do not pass parameters as JSON, instead pass the string to the tool as is.
    
    Output will be the text response from the Google API.
    """

    class GoogleGetCalendarsToolInputSchema(BaseModel):
        syncToken: Optional[str] = Field(description="The syncToken of the calendars")
        showHidden: Optional[bool] = Field(description="Whether to show hidden calendars")
        showDeleted: Optional[bool] = Field(description="Whether to show deleted calendars")
        pageToken: Optional[str] = Field(description="The pageToken of the calendars")
        
    input_schema = GoogleGetCalendarsToolInputSchema

    client: AuthenticatedClient
    user_id: str
    llm: Optional[BaseLanguageModel]
    parse_parameters: Optional[Callable[[str], GoogleGetCalendarsToolInputSchema]] = None
    parse_output: Optional[Callable[[GoogleCalendars], str]] = None
    
    def _parse_parameters(self, _query: str) -> GoogleGetCalendarsToolInputSchema:
        """Parse the input passed to the tool when running."""
        parser = PydanticOutputParser(pydantic_object=self.GoogleGetCalendarsToolInputSchema)
        prompt = PromptTemplate(
            template="""Parse the provided input string.
            For values that are not marked as required, if no value is provided, return a null value.
            {format_instructions}
            Here is the input:
            {query}""",
            input_variables=["query"],
            partial_variables={"format_instructions": parser.get_format_instructions()},
        )
        _input = prompt.format_prompt(query=_query)
        if self.llm is None:
            return "Error: No language model provided for input parsing."
        output = self.llm(_input.to_string())
        return parser.parse(output)
    
    def _run(self, query: str, run_manager: Optional[CallbackManagerForToolRun] = None) -> str:
        """Run the tool."""
        
        # If no custom input parameter parser is passed, use the default one
        if self.parse_parameters is None:
            parsed_parameters = self._parse_parameters(query)
        else: 
            parsed_parameters = self.parse_parameters(query)
        
        response: Response[List["ReturnedAccountDto"]] = find_all_accounts_for_user.sync_detailed(
            client=self.client, user_id=self.user_id
        )
        if not 200 <= response.status_code < 300:
            return "Error pulling the user's Google account."
        accounts: List["ReturnedAccountDto"] = response.parsed

        account_id = ""
        for account in accounts:
            if account.source == "google":
                account_id = account.id
        if account_id == "":
            return "The user has not connected their Google account; you should try authenticating Google first."
        else:
            response: Response[GoogleCalendars] = google_controller_calendars.sync_detailed(
                client=self.client,
                account_id=account_id,
                sync_token=parsed_parameters.syncToken,
                show_hidden=parsed_parameters.showHidden,
                show_deleted=parsed_parameters.showDeleted,
                page_token=parsed_parameters.pageToken
            )
            if not 200 <= response.status_code < 300:
                return "Error when attempting to get the user's Google calendars."

            if self.parse_output is None:
                return json.dumps(response.parsed.to_dict())
            else:
                return self.parse_output(response.parsed, query)

    # TODO implement async version
    async def _arun(
        self,
        query: str,
        run_manager: Optional[AsyncCallbackManagerForToolRun] = None,
    ) -> str:
        """Run the tool asynchronously."""
        return await self._run(query, run_manager)


class GoogleGetCalendarTool(BaseTool):
    name = "google_get_calendar_tool"
    description = """Useful for when you need to get a user's Google calendar.
    
    Input should be a string containing the question that you want to know the answer to.
    Do not pass parameters as JSON, instead pass the string to the tool as is.
    
    Output will be the text response from the Google API.
    """

    class GoogleGetCalendarToolInputSchema(BaseModel):
        calendarId: str = Field(description="The id of the calendar")
        
    input_schema = GoogleGetCalendarToolInputSchema

    client: AuthenticatedClient
    user_id: str
    llm: Optional[BaseLanguageModel]
    parse_parameters: Optional[Callable[[str], GoogleGetCalendarToolInputSchema]] = None
    parse_output: Optional[Callable[[GoogleCalendar], str]] = None
    
    def _parse_parameters(self, _query: str) -> GoogleGetCalendarToolInputSchema:
        """Parse the input passed to the tool when running."""
        parser = PydanticOutputParser(pydantic_object=self.GoogleGetCalendarToolInputSchema)
        prompt = PromptTemplate(
            template="""Parse the provided input string.
            For values that are not marked as required, if no value is provided, return a null value.
            {format_instructions}
            Here is the input:
            {query}""",
            input_variables=["query"],
            partial_variables={"format_instructions": parser.get_format_instructions()},
        )
        _input = prompt.format_prompt(query=_query)
        if self.llm is None:
            return "Error: No language model provided for input parsing."
        output = self.llm(_input.to_string())
        return parser.parse(output)
    
    def _run(self, query: str, run_manager: Optional[CallbackManagerForToolRun] = None) -> str:
        """Run the tool."""
        
        # If no custom input parameter parser is passed, use the default one
        if self.parse_parameters is None:
            parsed_parameters = self._parse_parameters(query)
        else: 
            parsed_parameters = self.parse_parameters(query)
        
        response: Response[List["ReturnedAccountDto"]] = find_all_accounts_for_user.sync_detailed(
            client=self.client, user_id=self.user_id
        )
        if not 200 <= response.status_code < 300:
            return "Error pulling the user's Google account."
        accounts: List["ReturnedAccountDto"] = response.parsed

        account_id = ""
        for account in accounts:
            if account.source == "google":
                account_id = account.id
        if account_id == "":
            return "The user has not connected their Google account; you should try authenticating Google first."
        else:
            response: Response[GoogleCalendar] = google_controller_calendar.sync_detailed(
                client=self.client,
                account_id=account_id,
                calendar_id=parsed_parameters.calendarId
            )
            if not 200 <= response.status_code < 300:
                return "Error when attempting to get the user's Google calendar."

            if self.parse_output is None:
                return json.dumps(response.parsed.to_dict())
            else:
                return self.parse_output(response.parsed, query)

    # TODO implement async version
    async def _arun(
        self,
        query: str,
        run_manager: Optional[AsyncCallbackManagerForToolRun] = None,
    ) -> str:
        """Run the tool asynchronously."""
        return await self._run(query, run_manager)


class GoogleGetEventsTool(BaseTool):
    name = "google_get_events_tool"
    description = """Useful for when you need to get a user's Google events.
    
    Input should be a string containing the question that you want to know the answer to.
    Do not pass parameters as JSON, instead pass the string to the tool as is.
    
    Output will be the text response from the Google API.
    """

    class GoogleGetEventsToolInputSchema(BaseModel):
        iCalUID: Optional[str] = Field(description="The iCal UID")
        syncToken: Optional[str] = Field(description="The sync token")
        updatedMin: Optional[str] = Field(description="The updated min")
        timeZone: Optional[str] = Field(description="The time zone")
        timeMin: Optional[str] = Field(description="The time min")
        timeMax: Optional[str] = Field(description="The time max")
        singleEvents: Optional[bool] = Field(description="Whether to show single events")
        showDeleted: Optional[bool] = Field(description="Whether to show deleted")
        q: Optional[str] = Field(description="The query")
        pageToken: Optional[str] = Field(description="The page token")
        orderBy: Optional[str] = Field(description="The order by")
        maxAttendees: Optional[int] = Field(description="The max attendees")
        calendarId: Optional[str] = Field(description="The id of the calendar")
        
    input_schema = GoogleGetEventsToolInputSchema

    client: AuthenticatedClient
    user_id: str
    llm: Optional[BaseLanguageModel]
    parse_parameters: Optional[Callable[[str], GoogleGetEventsToolInputSchema]] = None
    parse_output: Optional[Callable[[GoogleEvents], str]] = None
    
    def _parse_parameters(self, _query: str) -> GoogleGetEventsToolInputSchema:
        """Parse the input passed to the tool when running."""
        parser = PydanticOutputParser(pydantic_object=self.GoogleGetEventsToolInputSchema)
        prompt = PromptTemplate(
            template="""Parse the provided input string.
            For values that are not marked as required, if no value is provided, return a null value.
            {format_instructions}
            Here is the input:
            {query}""",
            input_variables=["query"],
            partial_variables={"format_instructions": parser.get_format_instructions()},
        )
        _input = prompt.format_prompt(query=_query)
        if self.llm is None:
            return "Error: No language model provided for input parsing."
        output = self.llm(_input.to_string())
        return parser.parse(output)
    
    def _run(self, query: str, run_manager: Optional[CallbackManagerForToolRun] = None) -> str:
        """Run the tool."""
        
        # If no custom input parameter parser is passed, use the default one
        if self.parse_parameters is None:
            parsed_parameters = self._parse_parameters(query)
        else: 
            parsed_parameters = self.parse_parameters(query)
        
        response: Response[List["ReturnedAccountDto"]] = find_all_accounts_for_user.sync_detailed(
            client=self.client, user_id=self.user_id
        )
        if not 200 <= response.status_code < 300:
            return "Error pulling the user's Google account."
        accounts: List["ReturnedAccountDto"] = response.parsed

        account_id = ""
        for account in accounts:
            if account.source == "google":
                account_id = account.id
        if account_id == "":
            return "The user has not connected their Google account; you should try authenticating Google first."
        else:
            response: Response[GoogleEvents] = google_controller_events.sync_detailed(
                client=self.client,
                account_id=account_id,
                i_cal_uid=parsed_parameters.iCalUID,
                sync_token=parsed_parameters.syncToken,
                updated_min=parsed_parameters.updatedMin,
                time_zone=parsed_parameters.timeZone,
                time_min=parsed_parameters.timeMin,
                time_max=parsed_parameters.timeMax,
                single_events=parsed_parameters.singleEvents,
                show_deleted=parsed_parameters.showDeleted,
                q=parsed_parameters.q,
                page_token=parsed_parameters.pageToken,
                order_by=parsed_parameters.orderBy,
                max_attendees=parsed_parameters.maxAttendees,
                calendar_id=parsed_parameters.calendarId
            )
            if not 200 <= response.status_code < 300:
                return "Error when attempting to get the user's Google events."

            if self.parse_output is None:
                return json.dumps(response.parsed.to_dict())
            else:
                return self.parse_output(response.parsed, query)

    # TODO implement async version
    async def _arun(
        self,
        query: str,
        run_manager: Optional[AsyncCallbackManagerForToolRun] = None,
    ) -> str:
        """Run the tool asynchronously."""
        return await self._run(query, run_manager)


class GoogleGetEventTool(BaseTool):
    name = "google_get_event_tool"
    description = """Useful for when you need to get a user's Google event.
    
    Input should be a string containing the question that you want to know the answer to.
    Do not pass parameters as JSON, instead pass the string to the tool as is.
    
    Output will be the text response from the Google API.
    """

    class GoogleGetEventToolInputSchema(BaseModel):
        timeZone: Optional[str] = Field(description="The timeZone of the event")
        eventId: str = Field(description="The id of the event")
        calendarId: str = Field(description="The calendarId of the calendar which contains the event")
        
    input_schema = GoogleGetEventToolInputSchema

    client: AuthenticatedClient
    user_id: str
    llm: Optional[BaseLanguageModel]
    parse_parameters: Optional[Callable[[str], GoogleGetEventToolInputSchema]] = None
    parse_output: Optional[Callable[[GoogleEvent], str]] = None
    
    def _parse_parameters(self, _query: str) -> GoogleGetEventToolInputSchema:
        """Parse the input passed to the tool when running."""
        parser = PydanticOutputParser(pydantic_object=self.GoogleGetEventToolInputSchema)
        prompt = PromptTemplate(
            template="""Parse the provided input string.
            For values that are not marked as required, if no value is provided, return a null value.
            {format_instructions}
            Here is the input:
            {query}""",
            input_variables=["query"],
            partial_variables={"format_instructions": parser.get_format_instructions()},
        )
        _input = prompt.format_prompt(query=_query)
        if self.llm is None:
            return "Error: No language model provided for input parsing."
        output = self.llm(_input.to_string())
        return parser.parse(output)
    
    def _run(self, query: str, run_manager: Optional[CallbackManagerForToolRun] = None) -> str:
        """Run the tool."""
        
        # If no custom input parameter parser is passed, use the default one
        if self.parse_parameters is None:
            parsed_parameters = self._parse_parameters(query)
        else: 
            parsed_parameters = self.parse_parameters(query)
        
        response: Response[List["ReturnedAccountDto"]] = find_all_accounts_for_user.sync_detailed(
            client=self.client, user_id=self.user_id
        )
        if not 200 <= response.status_code < 300:
            return "Error pulling the user's Google account."
        accounts: List["ReturnedAccountDto"] = response.parsed

        account_id = ""
        for account in accounts:
            if account.source == "google":
                account_id = account.id
        if account_id == "":
            return "The user has not connected their Google account; you should try authenticating Google first."
        else:
            response: Response[GoogleEvent] = google_controller_event.sync_detailed(
                client=self.client,
                account_id=account_id,
                time_zone=parsed_parameters.timeZone,
                event_id=parsed_parameters.eventId,
                calendar_id=parsed_parameters.calendarId
            )
            if not 200 <= response.status_code < 300:
                return "Error when attempting to get the user's Google event."

            if self.parse_output is None:
                return json.dumps(response.parsed.to_dict())
            else:
                return self.parse_output(response.parsed, query)

    # TODO implement async version
    async def _arun(
        self,
        query: str,
        run_manager: Optional[AsyncCallbackManagerForToolRun] = None,
    ) -> str:
        """Run the tool asynchronously."""
        return await self._run(query, run_manager)




from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.google_thread_messages_item_payload import GoogleThreadMessagesItemPayload


T = TypeVar("T", bound="GoogleThreadMessagesItem")


@attr.s(auto_attribs=True)
class GoogleThreadMessagesItem:
    """
    Attributes:
        id (Union[Unset, str]): The id of the message
        thread_id (Union[Unset, str]): The threadId of the message
        label_ids (Union[Unset, List[str]]): The labelIds of the message
        snippet (Union[Unset, str]): The snippet of the message
        history_id (Union[Unset, str]): The historyId of the message
        internal_date (Union[Unset, str]): The internalDate of the message
        payload (Union[Unset, GoogleThreadMessagesItemPayload]): The payload of the message
        size_estimate (Union[Unset, float]): The sizeEstimate of the message
    """

    id: Union[Unset, str] = UNSET
    thread_id: Union[Unset, str] = UNSET
    label_ids: Union[Unset, List[str]] = UNSET
    snippet: Union[Unset, str] = UNSET
    history_id: Union[Unset, str] = UNSET
    internal_date: Union[Unset, str] = UNSET
    payload: Union[Unset, "GoogleThreadMessagesItemPayload"] = UNSET
    size_estimate: Union[Unset, float] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        thread_id = self.thread_id
        label_ids: Union[Unset, List[str]] = UNSET
        if not isinstance(self.label_ids, Unset):
            label_ids = self.label_ids

        snippet = self.snippet
        history_id = self.history_id
        internal_date = self.internal_date
        payload: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.payload, Unset):
            payload = self.payload.to_dict()

        size_estimate = self.size_estimate

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if thread_id is not UNSET:
            field_dict["threadId"] = thread_id
        if label_ids is not UNSET:
            field_dict["labelIds"] = label_ids
        if snippet is not UNSET:
            field_dict["snippet"] = snippet
        if history_id is not UNSET:
            field_dict["historyId"] = history_id
        if internal_date is not UNSET:
            field_dict["internalDate"] = internal_date
        if payload is not UNSET:
            field_dict["payload"] = payload
        if size_estimate is not UNSET:
            field_dict["sizeEstimate"] = size_estimate

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.google_thread_messages_item_payload import GoogleThreadMessagesItemPayload

        d = src_dict.copy()
        id = d.pop("id", UNSET) or UNSET

        thread_id = d.pop("threadId", UNSET) or UNSET

        label_ids = cast(List[str], d.pop("labelIds", UNSET) or UNSET)

        snippet = d.pop("snippet", UNSET) or UNSET

        history_id = d.pop("historyId", UNSET) or UNSET

        internal_date = d.pop("internalDate", UNSET) or UNSET

        _payload = d.pop("payload", UNSET) or UNSET
        payload: Union[Unset, GoogleThreadMessagesItemPayload]
        if isinstance(_payload, Unset):
            payload = UNSET
        else:
            payload = GoogleThreadMessagesItemPayload.from_dict(_payload)

        size_estimate = d.pop("sizeEstimate", UNSET) or UNSET

        google_thread_messages_item = cls(
            id=id,
            thread_id=thread_id,
            label_ids=label_ids,
            snippet=snippet,
            history_id=history_id,
            internal_date=internal_date,
            payload=payload,
            size_estimate=size_estimate,
        )

        google_thread_messages_item.additional_properties = d
        return google_thread_messages_item

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.google_label_color import GoogleLabelColor


T = TypeVar("T", bound="GoogleLabel")


@attr.s(auto_attribs=True)
class GoogleLabel:
    """Your google label

    Attributes:
        id (Union[Unset, str]): The id of the label
        name (Union[Unset, str]): The name of the label
        message_list_visibility (Union[Unset, str]): The messageListVisibility of the label
        label_list_visibility (Union[Unset, str]): The labelListVisibility of the label
        type (Union[Unset, str]): The type of the label
        messages_total (Union[Unset, float]): The total messages for the label
        messages_unread (Union[Unset, float]): The total unread messages for the label
        threads_total (Union[Unset, float]): The total threads for the label
        threads_unread (Union[Unset, float]): The total unread threads for the label
        color (Union[Unset, GoogleLabelColor]): The color of the label
    """

    id: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    message_list_visibility: Union[Unset, str] = UNSET
    label_list_visibility: Union[Unset, str] = UNSET
    type: Union[Unset, str] = UNSET
    messages_total: Union[Unset, float] = UNSET
    messages_unread: Union[Unset, float] = UNSET
    threads_total: Union[Unset, float] = UNSET
    threads_unread: Union[Unset, float] = UNSET
    color: Union[Unset, "GoogleLabelColor"] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        name = self.name
        message_list_visibility = self.message_list_visibility
        label_list_visibility = self.label_list_visibility
        type = self.type
        messages_total = self.messages_total
        messages_unread = self.messages_unread
        threads_total = self.threads_total
        threads_unread = self.threads_unread
        color: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.color, Unset):
            color = self.color.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if message_list_visibility is not UNSET:
            field_dict["messageListVisibility"] = message_list_visibility
        if label_list_visibility is not UNSET:
            field_dict["labelListVisibility"] = label_list_visibility
        if type is not UNSET:
            field_dict["type"] = type
        if messages_total is not UNSET:
            field_dict["messagesTotal"] = messages_total
        if messages_unread is not UNSET:
            field_dict["messagesUnread"] = messages_unread
        if threads_total is not UNSET:
            field_dict["threadsTotal"] = threads_total
        if threads_unread is not UNSET:
            field_dict["threadsUnread"] = threads_unread
        if color is not UNSET:
            field_dict["color"] = color

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.google_label_color import GoogleLabelColor

        d = src_dict.copy()
        id = d.pop("id", UNSET) or UNSET

        name = d.pop("name", UNSET) or UNSET

        message_list_visibility = d.pop("messageListVisibility", UNSET) or UNSET

        label_list_visibility = d.pop("labelListVisibility", UNSET) or UNSET

        type = d.pop("type", UNSET) or UNSET

        messages_total = d.pop("messagesTotal", UNSET) or UNSET

        messages_unread = d.pop("messagesUnread", UNSET) or UNSET

        threads_total = d.pop("threadsTotal", UNSET) or UNSET

        threads_unread = d.pop("threadsUnread", UNSET) or UNSET

        _color = d.pop("color", UNSET) or UNSET
        color: Union[Unset, GoogleLabelColor]
        if isinstance(_color, Unset):
            color = UNSET
        else:
            color = GoogleLabelColor.from_dict(_color)

        google_label = cls(
            id=id,
            name=name,
            message_list_visibility=message_list_visibility,
            label_list_visibility=label_list_visibility,
            type=type,
            messages_total=messages_total,
            messages_unread=messages_unread,
            threads_total=threads_total,
            threads_unread=threads_unread,
            color=color,
        )

        google_label.additional_properties = d
        return google_label

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.google_event_conference_data_create_request_conference_solution_key import (
        GoogleEventConferenceDataCreateRequestConferenceSolutionKey,
    )
    from ..models.google_event_conference_data_create_request_status import GoogleEventConferenceDataCreateRequestStatus


T = TypeVar("T", bound="GoogleEventConferenceDataCreateRequest")


@attr.s(auto_attribs=True)
class GoogleEventConferenceDataCreateRequest:
    """The createRequest of the conferenceData

    Attributes:
        request_id (Union[Unset, str]): The requestId of the createRequest
        conference_solution_key (Union[Unset, GoogleEventConferenceDataCreateRequestConferenceSolutionKey]): The
            conferenceSolutionKey of the createRequest
        status (Union[Unset, GoogleEventConferenceDataCreateRequestStatus]): The status of the createRequest
    """

    request_id: Union[Unset, str] = UNSET
    conference_solution_key: Union[Unset, "GoogleEventConferenceDataCreateRequestConferenceSolutionKey"] = UNSET
    status: Union[Unset, "GoogleEventConferenceDataCreateRequestStatus"] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        request_id = self.request_id
        conference_solution_key: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.conference_solution_key, Unset):
            conference_solution_key = self.conference_solution_key.to_dict()

        status: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if request_id is not UNSET:
            field_dict["requestId"] = request_id
        if conference_solution_key is not UNSET:
            field_dict["conferenceSolutionKey"] = conference_solution_key
        if status is not UNSET:
            field_dict["status"] = status

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.google_event_conference_data_create_request_conference_solution_key import (
            GoogleEventConferenceDataCreateRequestConferenceSolutionKey,
        )
        from ..models.google_event_conference_data_create_request_status import (
            GoogleEventConferenceDataCreateRequestStatus,
        )

        d = src_dict.copy()
        request_id = d.pop("requestId", UNSET) or UNSET

        _conference_solution_key = d.pop("conferenceSolutionKey", UNSET) or UNSET
        conference_solution_key: Union[Unset, GoogleEventConferenceDataCreateRequestConferenceSolutionKey]
        if isinstance(_conference_solution_key, Unset):
            conference_solution_key = UNSET
        else:
            conference_solution_key = GoogleEventConferenceDataCreateRequestConferenceSolutionKey.from_dict(
                _conference_solution_key
            )

        _status = d.pop("status", UNSET) or UNSET
        status: Union[Unset, GoogleEventConferenceDataCreateRequestStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = GoogleEventConferenceDataCreateRequestStatus.from_dict(_status)

        google_event_conference_data_create_request = cls(
            request_id=request_id,
            conference_solution_key=conference_solution_key,
            status=status,
        )

        google_event_conference_data_create_request.additional_properties = d
        return google_event_conference_data_create_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.confirmation_status import ConfirmationStatus
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.returned_confirm_dto_params import ReturnedConfirmDtoParams


T = TypeVar("T", bound="ReturnedConfirmDto")


@attr.s(auto_attribs=True)
class ReturnedConfirmDto:
    """
    Attributes:
        id (str):
        type (str):
        action (str):
        status (ConfirmationStatus):
        delivery_channel (str):
        delivery_address (Union[Unset, str]):
        used_at (Union[Unset, datetime.datetime]):
        responded_at (Union[Unset, datetime.datetime]):
        params (Union[Unset, ReturnedConfirmDtoParams]):
    """

    id: str
    type: str
    action: str
    status: ConfirmationStatus
    delivery_channel: str
    delivery_address: Union[Unset, str] = UNSET
    used_at: Union[Unset, datetime.datetime] = UNSET
    responded_at: Union[Unset, datetime.datetime] = UNSET
    params: Union[Unset, "ReturnedConfirmDtoParams"] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        type = self.type
        action = self.action
        status = self.status.value

        delivery_channel = self.delivery_channel
        delivery_address = self.delivery_address
        used_at: Union[Unset, str] = UNSET
        if not isinstance(self.used_at, Unset):
            used_at = self.used_at.isoformat()

        responded_at: Union[Unset, str] = UNSET
        if not isinstance(self.responded_at, Unset):
            responded_at = self.responded_at.isoformat()

        params: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.params, Unset):
            params = self.params.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "type": type,
                "action": action,
                "status": status,
                "deliveryChannel": delivery_channel,
            }
        )
        if delivery_address is not UNSET:
            field_dict["deliveryAddress"] = delivery_address
        if used_at is not UNSET:
            field_dict["usedAt"] = used_at
        if responded_at is not UNSET:
            field_dict["respondedAt"] = responded_at
        if params is not UNSET:
            field_dict["params"] = params

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.returned_confirm_dto_params import ReturnedConfirmDtoParams

        d = src_dict.copy()
        id = d.pop("id")

        type = d.pop("type")

        action = d.pop("action")

        status = ConfirmationStatus(d.pop("status"))

        delivery_channel = d.pop("deliveryChannel")

        delivery_address = d.pop("deliveryAddress", UNSET) or UNSET

        _used_at = d.pop("usedAt", UNSET) or UNSET
        used_at: Union[Unset, datetime.datetime]
        if isinstance(_used_at, Unset):
            used_at = UNSET
        else:
            used_at = isoparse(_used_at)

        _responded_at = d.pop("respondedAt", UNSET) or UNSET
        responded_at: Union[Unset, datetime.datetime]
        if isinstance(_responded_at, Unset):
            responded_at = UNSET
        else:
            responded_at = isoparse(_responded_at)

        _params = d.pop("params", UNSET) or UNSET
        params: Union[Unset, ReturnedConfirmDtoParams]
        if isinstance(_params, Unset):
            params = UNSET
        else:
            params = ReturnedConfirmDtoParams.from_dict(_params)

        returned_confirm_dto = cls(
            id=id,
            type=type,
            action=action,
            status=status,
            delivery_channel=delivery_channel,
            delivery_address=delivery_address,
            used_at=used_at,
            responded_at=responded_at,
            params=params,
        )

        returned_confirm_dto.additional_properties = d
        return returned_confirm_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

import attr

if TYPE_CHECKING:
    from ..models.source_data import SourceData


T = TypeVar("T", bound="ClientUsageData")


@attr.s(auto_attribs=True)
class ClientUsageData:
    """
    Attributes:
        total_requests (float):
        total_successes (float):
        total_failures (float):
        unique_users (float):
        source_data (List['SourceData']):
    """

    total_requests: float
    total_successes: float
    total_failures: float
    unique_users: float
    source_data: List["SourceData"]
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        total_requests = self.total_requests
        total_successes = self.total_successes
        total_failures = self.total_failures
        unique_users = self.unique_users
        source_data = []
        for source_data_item_data in self.source_data:
            source_data_item = source_data_item_data.to_dict()

            source_data.append(source_data_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "totalRequests": total_requests,
                "totalSuccesses": total_successes,
                "totalFailures": total_failures,
                "uniqueUsers": unique_users,
                "sourceData": source_data,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.source_data import SourceData

        d = src_dict.copy()
        total_requests = d.pop("totalRequests")

        total_successes = d.pop("totalSuccesses")

        total_failures = d.pop("totalFailures")

        unique_users = d.pop("uniqueUsers")

        source_data = []
        _source_data = d.pop("sourceData")
        for source_data_item_data in _source_data:
            source_data_item = SourceData.from_dict(source_data_item_data)

            source_data.append(source_data_item)

        client_usage_data = cls(
            total_requests=total_requests,
            total_successes=total_successes,
            total_failures=total_failures,
            unique_users=unique_users,
            source_data=source_data,
        )

        client_usage_data.additional_properties = d
        return client_usage_data

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.google_thread_messages_item import GoogleThreadMessagesItem


T = TypeVar("T", bound="GoogleThread")


@attr.s(auto_attribs=True)
class GoogleThread:
    """Your google thread

    Attributes:
        id (Union[Unset, str]): The id of the thread
        snippet (Union[Unset, str]): The snippet of the thread
        history_id (Union[Unset, str]): The historyId of the thread
        messages (Union[Unset, List['GoogleThreadMessagesItem']]): A list of the messages in the thread
    """

    id: Union[Unset, str] = UNSET
    snippet: Union[Unset, str] = UNSET
    history_id: Union[Unset, str] = UNSET
    messages: Union[Unset, List["GoogleThreadMessagesItem"]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        snippet = self.snippet
        history_id = self.history_id
        messages: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.messages, Unset):
            messages = []
            for messages_item_data in self.messages:
                messages_item = messages_item_data.to_dict()

                messages.append(messages_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if snippet is not UNSET:
            field_dict["snippet"] = snippet
        if history_id is not UNSET:
            field_dict["historyId"] = history_id
        if messages is not UNSET:
            field_dict["messages"] = messages

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.google_thread_messages_item import GoogleThreadMessagesItem

        d = src_dict.copy()
        id = d.pop("id", UNSET) or UNSET

        snippet = d.pop("snippet", UNSET) or UNSET

        history_id = d.pop("historyId", UNSET) or UNSET

        messages = []
        _messages = d.pop("messages", UNSET) or UNSET
        for messages_item_data in _messages or []:
            messages_item = GoogleThreadMessagesItem.from_dict(messages_item_data)

            messages.append(messages_item)

        google_thread = cls(
            id=id,
            snippet=snippet,
            history_id=history_id,
            messages=messages,
        )

        google_thread.additional_properties = d
        return google_thread

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="GoogleEventAttachmentsItem")


@attr.s(auto_attribs=True)
class GoogleEventAttachmentsItem:
    """An attachment

    Attributes:
        file_url (Union[Unset, str]): The fileUrl of the attachment
        title (Union[Unset, str]): The title of the attachment
        mime_type (Union[Unset, str]): The mimeType of the attachment
        icon_link (Union[Unset, str]): The iconLink of the attachment
        file_id (Union[Unset, str]): The fileId of the attachment
    """

    file_url: Union[Unset, str] = UNSET
    title: Union[Unset, str] = UNSET
    mime_type: Union[Unset, str] = UNSET
    icon_link: Union[Unset, str] = UNSET
    file_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        file_url = self.file_url
        title = self.title
        mime_type = self.mime_type
        icon_link = self.icon_link
        file_id = self.file_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if file_url is not UNSET:
            field_dict["fileUrl"] = file_url
        if title is not UNSET:
            field_dict["title"] = title
        if mime_type is not UNSET:
            field_dict["mimeType"] = mime_type
        if icon_link is not UNSET:
            field_dict["iconLink"] = icon_link
        if file_id is not UNSET:
            field_dict["fileId"] = file_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        file_url = d.pop("fileUrl", UNSET) or UNSET

        title = d.pop("title", UNSET) or UNSET

        mime_type = d.pop("mimeType", UNSET) or UNSET

        icon_link = d.pop("iconLink", UNSET) or UNSET

        file_id = d.pop("fileId", UNSET) or UNSET

        google_event_attachments_item = cls(
            file_url=file_url,
            title=title,
            mime_type=mime_type,
            icon_link=icon_link,
            file_id=file_id,
        )

        google_event_attachments_item.additional_properties = d
        return google_event_attachments_item

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.google_parsed_messages_messages_item_attachments_item import (
        GoogleParsedMessagesMessagesItemAttachmentsItem,
    )
    from ..models.google_parsed_messages_messages_item_headers import GoogleParsedMessagesMessagesItemHeaders


T = TypeVar("T", bound="GoogleParsedMessagesMessagesItem")


@attr.s(auto_attribs=True)
class GoogleParsedMessagesMessagesItem:
    """
    Attributes:
        id (Union[Unset, str]): The id of the message
        thread_id (Union[Unset, str]): The threadId of the message
        label_ids (Union[Unset, List[str]]): The labelIds of the message
        headers (Union[Unset, GoogleParsedMessagesMessagesItemHeaders]): The headers of the message
        body (Union[Unset, str]): The body of the message
        attachments (Union[Unset, List['GoogleParsedMessagesMessagesItemAttachmentsItem']]): The attachments of the
            message
    """

    id: Union[Unset, str] = UNSET
    thread_id: Union[Unset, str] = UNSET
    label_ids: Union[Unset, List[str]] = UNSET
    headers: Union[Unset, "GoogleParsedMessagesMessagesItemHeaders"] = UNSET
    body: Union[Unset, str] = UNSET
    attachments: Union[Unset, List["GoogleParsedMessagesMessagesItemAttachmentsItem"]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        thread_id = self.thread_id
        label_ids: Union[Unset, List[str]] = UNSET
        if not isinstance(self.label_ids, Unset):
            label_ids = self.label_ids

        headers: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.headers, Unset):
            headers = self.headers.to_dict()

        body = self.body
        attachments: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.attachments, Unset):
            attachments = []
            for attachments_item_data in self.attachments:
                attachments_item = attachments_item_data.to_dict()

                attachments.append(attachments_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if thread_id is not UNSET:
            field_dict["threadId"] = thread_id
        if label_ids is not UNSET:
            field_dict["labelIds"] = label_ids
        if headers is not UNSET:
            field_dict["headers"] = headers
        if body is not UNSET:
            field_dict["body"] = body
        if attachments is not UNSET:
            field_dict["attachments"] = attachments

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.google_parsed_messages_messages_item_attachments_item import (
            GoogleParsedMessagesMessagesItemAttachmentsItem,
        )
        from ..models.google_parsed_messages_messages_item_headers import GoogleParsedMessagesMessagesItemHeaders

        d = src_dict.copy()
        id = d.pop("id", UNSET) or UNSET

        thread_id = d.pop("threadId", UNSET) or UNSET

        label_ids = cast(List[str], d.pop("labelIds", UNSET) or UNSET)

        _headers = d.pop("headers", UNSET) or UNSET
        headers: Union[Unset, GoogleParsedMessagesMessagesItemHeaders]
        if isinstance(_headers, Unset):
            headers = UNSET
        else:
            headers = GoogleParsedMessagesMessagesItemHeaders.from_dict(_headers)

        body = d.pop("body", UNSET) or UNSET

        attachments = []
        _attachments = d.pop("attachments", UNSET) or UNSET
        for attachments_item_data in _attachments or []:
            attachments_item = GoogleParsedMessagesMessagesItemAttachmentsItem.from_dict(attachments_item_data)

            attachments.append(attachments_item)

        google_parsed_messages_messages_item = cls(
            id=id,
            thread_id=thread_id,
            label_ids=label_ids,
            headers=headers,
            body=body,
            attachments=attachments,
        )

        google_parsed_messages_messages_item.additional_properties = d
        return google_parsed_messages_messages_item

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="GoogleLabelsLabelsItem")


@attr.s(auto_attribs=True)
class GoogleLabelsLabelsItem:
    """
    Attributes:
        id (Union[Unset, str]): The id of the label
        name (Union[Unset, str]): The name of the label
        message_list_visibility (Union[Unset, str]): The messageListVisibility of the label
        label_list_visibility (Union[Unset, str]): The labelListVisibility of the label
        type (Union[Unset, str]): The type of the label
    """

    id: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    message_list_visibility: Union[Unset, str] = UNSET
    label_list_visibility: Union[Unset, str] = UNSET
    type: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        name = self.name
        message_list_visibility = self.message_list_visibility
        label_list_visibility = self.label_list_visibility
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if message_list_visibility is not UNSET:
            field_dict["messageListVisibility"] = message_list_visibility
        if label_list_visibility is not UNSET:
            field_dict["labelListVisibility"] = label_list_visibility
        if type is not UNSET:
            field_dict["type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET) or UNSET

        name = d.pop("name", UNSET) or UNSET

        message_list_visibility = d.pop("messageListVisibility", UNSET) or UNSET

        label_list_visibility = d.pop("labelListVisibility", UNSET) or UNSET

        type = d.pop("type", UNSET) or UNSET

        google_labels_labels_item = cls(
            id=id,
            name=name,
            message_list_visibility=message_list_visibility,
            label_list_visibility=label_list_visibility,
            type=type,
        )

        google_labels_labels_item.additional_properties = d
        return google_labels_labels_item

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="CreateUserDto")


@attr.s(auto_attribs=True)
class CreateUserDto:
    """
    Attributes:
        external_id (Union[Unset, str]):
        first_name (Union[Unset, str]):
        last_name (Union[Unset, str]):
        email (Union[Unset, str]):
    """

    external_id: Union[Unset, str] = UNSET
    first_name: Union[Unset, str] = UNSET
    last_name: Union[Unset, str] = UNSET
    email: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        external_id = self.external_id
        first_name = self.first_name
        last_name = self.last_name
        email = self.email

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if external_id is not UNSET:
            field_dict["externalId"] = external_id
        if first_name is not UNSET:
            field_dict["firstName"] = first_name
        if last_name is not UNSET:
            field_dict["lastName"] = last_name
        if email is not UNSET:
            field_dict["email"] = email

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        external_id = d.pop("externalId", UNSET) or UNSET

        first_name = d.pop("firstName", UNSET) or UNSET

        last_name = d.pop("lastName", UNSET) or UNSET

        email = d.pop("email", UNSET) or UNSET

        create_user_dto = cls(
            external_id=external_id,
            first_name=first_name,
            last_name=last_name,
            email=email,
        )

        create_user_dto.additional_properties = d
        return create_user_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="GoogleProfile")


@attr.s(auto_attribs=True)
class GoogleProfile:
    """A google profile

    Attributes:
        email_address (Union[Unset, str]): The email of the user
        messages_total (Union[Unset, float]): The total messages of the user
        threads_total (Union[Unset, float]): The total email threads of the user
        history_id (Union[Unset, str]): The historyId of the user
    """

    email_address: Union[Unset, str] = UNSET
    messages_total: Union[Unset, float] = UNSET
    threads_total: Union[Unset, float] = UNSET
    history_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        email_address = self.email_address
        messages_total = self.messages_total
        threads_total = self.threads_total
        history_id = self.history_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if email_address is not UNSET:
            field_dict["emailAddress"] = email_address
        if messages_total is not UNSET:
            field_dict["messagesTotal"] = messages_total
        if threads_total is not UNSET:
            field_dict["threadsTotal"] = threads_total
        if history_id is not UNSET:
            field_dict["historyId"] = history_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        email_address = d.pop("emailAddress", UNSET) or UNSET

        messages_total = d.pop("messagesTotal", UNSET) or UNSET

        threads_total = d.pop("threadsTotal", UNSET) or UNSET

        history_id = d.pop("historyId", UNSET) or UNSET

        google_profile = cls(
            email_address=email_address,
            messages_total=messages_total,
            threads_total=threads_total,
            history_id=history_id,
        )

        google_profile.additional_properties = d
        return google_profile

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="GoogleEventConferenceDataCreateRequestStatus")


@attr.s(auto_attribs=True)
class GoogleEventConferenceDataCreateRequestStatus:
    """The status of the createRequest

    Attributes:
        status_code (Union[Unset, str]): The statusCode of the status
    """

    status_code: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        status_code = self.status_code

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if status_code is not UNSET:
            field_dict["statusCode"] = status_code

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        status_code = d.pop("statusCode", UNSET) or UNSET

        google_event_conference_data_create_request_status = cls(
            status_code=status_code,
        )

        google_event_conference_data_create_request_status.additional_properties = d
        return google_event_conference_data_create_request_status

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class SourceType(str, Enum):
    AMAZON = "amazon"
    CASHAPP = "cashapp"
    DISCORD = "discord"
    DUNKIN = "dunkin"
    EMAIL = "email"
    ETHEREUM = "ethereum"
    FACEBOOK = "facebook"
    GCALENDAR = "gcalendar"
    GITHUB = "github"
    GMAIL = "gmail"
    GOOGLE = "google"
    LINKEDIN = "linkedin"
    POLYGON = "polygon"
    SHOPIFY = "shopify"
    SPOTIFY = "spotify"
    STACKOVERFLOW = "stackoverflow"
    TWITTER = "twitter"
    VENMO = "venmo"

    def __str__(self) -> str:
        return str(self.value)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.google_events_default_reminders_item import GoogleEventsDefaultRemindersItem
    from ..models.google_events_events_item import GoogleEventsEventsItem


T = TypeVar("T", bound="GoogleEvents")


@attr.s(auto_attribs=True)
class GoogleEvents:
    """Your google calendar's events

    Attributes:
        kind (Union[Unset, str]): The kind of the events
        etag (Union[Unset, str]): The etag of the events
        summary (Union[Unset, str]): The summary of the events
        description (Union[Unset, str]): The description of the events
        updated (Union[Unset, str]): The updated date of the events
        time_zone (Union[Unset, str]): The timeZone of the events
        access_role (Union[Unset, str]): The accessRole of the events
        default_reminders (Union[Unset, List['GoogleEventsDefaultRemindersItem']]): The defaultReminders of the events
        next_page_token (Union[Unset, str]): The nextPageToken of the events
        next_sync_token (Union[Unset, str]): The nextSyncToken of the events
        result_size_estimate (Union[Unset, float]): The resultSizeEstimate of the events
        events (Union[Unset, List['GoogleEventsEventsItem']]): The list of events
    """

    kind: Union[Unset, str] = UNSET
    etag: Union[Unset, str] = UNSET
    summary: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    updated: Union[Unset, str] = UNSET
    time_zone: Union[Unset, str] = UNSET
    access_role: Union[Unset, str] = UNSET
    default_reminders: Union[Unset, List["GoogleEventsDefaultRemindersItem"]] = UNSET
    next_page_token: Union[Unset, str] = UNSET
    next_sync_token: Union[Unset, str] = UNSET
    result_size_estimate: Union[Unset, float] = UNSET
    events: Union[Unset, List["GoogleEventsEventsItem"]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        kind = self.kind
        etag = self.etag
        summary = self.summary
        description = self.description
        updated = self.updated
        time_zone = self.time_zone
        access_role = self.access_role
        default_reminders: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.default_reminders, Unset):
            default_reminders = []
            for default_reminders_item_data in self.default_reminders:
                default_reminders_item = default_reminders_item_data.to_dict()

                default_reminders.append(default_reminders_item)

        next_page_token = self.next_page_token
        next_sync_token = self.next_sync_token
        result_size_estimate = self.result_size_estimate
        events: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.events, Unset):
            events = []
            for events_item_data in self.events:
                events_item = events_item_data.to_dict()

                events.append(events_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if kind is not UNSET:
            field_dict["kind"] = kind
        if etag is not UNSET:
            field_dict["etag"] = etag
        if summary is not UNSET:
            field_dict["summary"] = summary
        if description is not UNSET:
            field_dict["description"] = description
        if updated is not UNSET:
            field_dict["updated"] = updated
        if time_zone is not UNSET:
            field_dict["timeZone"] = time_zone
        if access_role is not UNSET:
            field_dict["accessRole"] = access_role
        if default_reminders is not UNSET:
            field_dict["defaultReminders"] = default_reminders
        if next_page_token is not UNSET:
            field_dict["nextPageToken"] = next_page_token
        if next_sync_token is not UNSET:
            field_dict["nextSyncToken"] = next_sync_token
        if result_size_estimate is not UNSET:
            field_dict["resultSizeEstimate"] = result_size_estimate
        if events is not UNSET:
            field_dict["events"] = events

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.google_events_default_reminders_item import GoogleEventsDefaultRemindersItem
        from ..models.google_events_events_item import GoogleEventsEventsItem

        d = src_dict.copy()
        kind = d.pop("kind", UNSET) or UNSET

        etag = d.pop("etag", UNSET) or UNSET

        summary = d.pop("summary", UNSET) or UNSET

        description = d.pop("description", UNSET) or UNSET

        updated = d.pop("updated", UNSET) or UNSET

        time_zone = d.pop("timeZone", UNSET) or UNSET

        access_role = d.pop("accessRole", UNSET) or UNSET

        default_reminders = []
        _default_reminders = d.pop("defaultReminders", UNSET) or UNSET
        for default_reminders_item_data in _default_reminders or []:
            default_reminders_item = GoogleEventsDefaultRemindersItem.from_dict(default_reminders_item_data)

            default_reminders.append(default_reminders_item)

        next_page_token = d.pop("nextPageToken", UNSET) or UNSET

        next_sync_token = d.pop("nextSyncToken", UNSET) or UNSET

        result_size_estimate = d.pop("resultSizeEstimate", UNSET) or UNSET

        events = []
        _events = d.pop("events", UNSET) or UNSET
        for events_item_data in _events or []:
            events_item = GoogleEventsEventsItem.from_dict(events_item_data)

            events.append(events_item)

        google_events = cls(
            kind=kind,
            etag=etag,
            summary=summary,
            description=description,
            updated=updated,
            time_zone=time_zone,
            access_role=access_role,
            default_reminders=default_reminders,
            next_page_token=next_page_token,
            next_sync_token=next_sync_token,
            result_size_estimate=result_size_estimate,
            events=events,
        )

        google_events.additional_properties = d
        return google_events

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="GoogleParsedDraftsDraftsItemAttachmentsItem")


@attr.s(auto_attribs=True)
class GoogleParsedDraftsDraftsItemAttachmentsItem:
    """An attachment of the draft

    Attributes:
        attachment_id (Union[Unset, str]): The attachmentId of the attachment
        mime_type (Union[Unset, str]): The mimeType of the attachment
        filename (Union[Unset, str]): The filename of the attachment
        content_type (Union[Unset, str]): The contentType of the attachment
        content_disposition (Union[Unset, str]): The contentDisposition of the attachment
        content_transfer_encoding (Union[Unset, str]): The contentTransferEncoding of the attachment
        size (Union[Unset, float]): The size of the attachment
    """

    attachment_id: Union[Unset, str] = UNSET
    mime_type: Union[Unset, str] = UNSET
    filename: Union[Unset, str] = UNSET
    content_type: Union[Unset, str] = UNSET
    content_disposition: Union[Unset, str] = UNSET
    content_transfer_encoding: Union[Unset, str] = UNSET
    size: Union[Unset, float] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        attachment_id = self.attachment_id
        mime_type = self.mime_type
        filename = self.filename
        content_type = self.content_type
        content_disposition = self.content_disposition
        content_transfer_encoding = self.content_transfer_encoding
        size = self.size

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if attachment_id is not UNSET:
            field_dict["attachmentId"] = attachment_id
        if mime_type is not UNSET:
            field_dict["mimeType"] = mime_type
        if filename is not UNSET:
            field_dict["filename"] = filename
        if content_type is not UNSET:
            field_dict["contentType"] = content_type
        if content_disposition is not UNSET:
            field_dict["contentDisposition"] = content_disposition
        if content_transfer_encoding is not UNSET:
            field_dict["contentTransferEncoding"] = content_transfer_encoding
        if size is not UNSET:
            field_dict["size"] = size

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        attachment_id = d.pop("attachmentId", UNSET) or UNSET

        mime_type = d.pop("mimeType", UNSET) or UNSET

        filename = d.pop("filename", UNSET) or UNSET

        content_type = d.pop("contentType", UNSET) or UNSET

        content_disposition = d.pop("contentDisposition", UNSET) or UNSET

        content_transfer_encoding = d.pop("contentTransferEncoding", UNSET) or UNSET

        size = d.pop("size", UNSET) or UNSET

        google_parsed_drafts_drafts_item_attachments_item = cls(
            attachment_id=attachment_id,
            mime_type=mime_type,
            filename=filename,
            content_type=content_type,
            content_disposition=content_disposition,
            content_transfer_encoding=content_transfer_encoding,
            size=size,
        )

        google_parsed_drafts_drafts_item_attachments_item.additional_properties = d
        return google_parsed_drafts_drafts_item_attachments_item

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.google_events_events_item_conference_data_conference_solution import (
        GoogleEventsEventsItemConferenceDataConferenceSolution,
    )
    from ..models.google_events_events_item_conference_data_create_request import (
        GoogleEventsEventsItemConferenceDataCreateRequest,
    )
    from ..models.google_events_events_item_conference_data_entry_points_item import (
        GoogleEventsEventsItemConferenceDataEntryPointsItem,
    )


T = TypeVar("T", bound="GoogleEventsEventsItemConferenceData")


@attr.s(auto_attribs=True)
class GoogleEventsEventsItemConferenceData:
    """The conferenceData of the event

    Attributes:
        create_request (Union[Unset, GoogleEventsEventsItemConferenceDataCreateRequest]): The createRequest of the
            conferenceData
        entry_points (Union[Unset, List['GoogleEventsEventsItemConferenceDataEntryPointsItem']]): The entryPoints of the
            conferenceData
        conference_solution (Union[Unset, GoogleEventsEventsItemConferenceDataConferenceSolution]): The
            conferenceSolution of the conferenceData
        conference_id (Union[Unset, str]): The conferenceId of the conferenceData
        signature (Union[Unset, str]): The signature of the conferenceData
        notes (Union[Unset, str]): The notes of the conferenceData
    """

    create_request: Union[Unset, "GoogleEventsEventsItemConferenceDataCreateRequest"] = UNSET
    entry_points: Union[Unset, List["GoogleEventsEventsItemConferenceDataEntryPointsItem"]] = UNSET
    conference_solution: Union[Unset, "GoogleEventsEventsItemConferenceDataConferenceSolution"] = UNSET
    conference_id: Union[Unset, str] = UNSET
    signature: Union[Unset, str] = UNSET
    notes: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        create_request: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.create_request, Unset):
            create_request = self.create_request.to_dict()

        entry_points: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.entry_points, Unset):
            entry_points = []
            for entry_points_item_data in self.entry_points:
                entry_points_item = entry_points_item_data.to_dict()

                entry_points.append(entry_points_item)

        conference_solution: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.conference_solution, Unset):
            conference_solution = self.conference_solution.to_dict()

        conference_id = self.conference_id
        signature = self.signature
        notes = self.notes

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if create_request is not UNSET:
            field_dict["createRequest"] = create_request
        if entry_points is not UNSET:
            field_dict["entryPoints"] = entry_points
        if conference_solution is not UNSET:
            field_dict["conferenceSolution"] = conference_solution
        if conference_id is not UNSET:
            field_dict["conferenceId"] = conference_id
        if signature is not UNSET:
            field_dict["signature"] = signature
        if notes is not UNSET:
            field_dict["notes"] = notes

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.google_events_events_item_conference_data_conference_solution import (
            GoogleEventsEventsItemConferenceDataConferenceSolution,
        )
        from ..models.google_events_events_item_conference_data_create_request import (
            GoogleEventsEventsItemConferenceDataCreateRequest,
        )
        from ..models.google_events_events_item_conference_data_entry_points_item import (
            GoogleEventsEventsItemConferenceDataEntryPointsItem,
        )

        d = src_dict.copy()
        _create_request = d.pop("createRequest", UNSET) or UNSET
        create_request: Union[Unset, GoogleEventsEventsItemConferenceDataCreateRequest]
        if isinstance(_create_request, Unset):
            create_request = UNSET
        else:
            create_request = GoogleEventsEventsItemConferenceDataCreateRequest.from_dict(_create_request)

        entry_points = []
        _entry_points = d.pop("entryPoints", UNSET) or UNSET
        for entry_points_item_data in _entry_points or []:
            entry_points_item = GoogleEventsEventsItemConferenceDataEntryPointsItem.from_dict(entry_points_item_data)

            entry_points.append(entry_points_item)

        _conference_solution = d.pop("conferenceSolution", UNSET) or UNSET
        conference_solution: Union[Unset, GoogleEventsEventsItemConferenceDataConferenceSolution]
        if isinstance(_conference_solution, Unset):
            conference_solution = UNSET
        else:
            conference_solution = GoogleEventsEventsItemConferenceDataConferenceSolution.from_dict(_conference_solution)

        conference_id = d.pop("conferenceId", UNSET) or UNSET

        signature = d.pop("signature", UNSET) or UNSET

        notes = d.pop("notes", UNSET) or UNSET

        google_events_events_item_conference_data = cls(
            create_request=create_request,
            entry_points=entry_points,
            conference_solution=conference_solution,
            conference_id=conference_id,
            signature=signature,
            notes=notes,
        )

        google_events_events_item_conference_data.additional_properties = d
        return google_events_events_item_conference_data

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.google_message_payload_parts_item_body import GoogleMessagePayloadPartsItemBody
    from ..models.google_message_payload_parts_item_headers_item import GoogleMessagePayloadPartsItemHeadersItem


T = TypeVar("T", bound="GoogleMessagePayloadPartsItem")


@attr.s(auto_attribs=True)
class GoogleMessagePayloadPartsItem:
    """
    Attributes:
        part_id (Union[Unset, str]): The partId of the part
        mime_type (Union[Unset, str]): The mimeType of the part
        filename (Union[Unset, str]): The filename of the part
        headers (Union[Unset, List['GoogleMessagePayloadPartsItemHeadersItem']]): The headers of the part
        body (Union[Unset, GoogleMessagePayloadPartsItemBody]): The body of the part
    """

    part_id: Union[Unset, str] = UNSET
    mime_type: Union[Unset, str] = UNSET
    filename: Union[Unset, str] = UNSET
    headers: Union[Unset, List["GoogleMessagePayloadPartsItemHeadersItem"]] = UNSET
    body: Union[Unset, "GoogleMessagePayloadPartsItemBody"] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        part_id = self.part_id
        mime_type = self.mime_type
        filename = self.filename
        headers: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.headers, Unset):
            headers = []
            for headers_item_data in self.headers:
                headers_item = headers_item_data.to_dict()

                headers.append(headers_item)

        body: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.body, Unset):
            body = self.body.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if part_id is not UNSET:
            field_dict["partId"] = part_id
        if mime_type is not UNSET:
            field_dict["mimeType"] = mime_type
        if filename is not UNSET:
            field_dict["filename"] = filename
        if headers is not UNSET:
            field_dict["headers"] = headers
        if body is not UNSET:
            field_dict["body"] = body

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.google_message_payload_parts_item_body import GoogleMessagePayloadPartsItemBody
        from ..models.google_message_payload_parts_item_headers_item import GoogleMessagePayloadPartsItemHeadersItem

        d = src_dict.copy()
        part_id = d.pop("partId", UNSET) or UNSET

        mime_type = d.pop("mimeType", UNSET) or UNSET

        filename = d.pop("filename", UNSET) or UNSET

        headers = []
        _headers = d.pop("headers", UNSET) or UNSET
        for headers_item_data in _headers or []:
            headers_item = GoogleMessagePayloadPartsItemHeadersItem.from_dict(headers_item_data)

            headers.append(headers_item)

        _body = d.pop("body", UNSET) or UNSET
        body: Union[Unset, GoogleMessagePayloadPartsItemBody]
        if isinstance(_body, Unset):
            body = UNSET
        else:
            body = GoogleMessagePayloadPartsItemBody.from_dict(_body)

        google_message_payload_parts_item = cls(
            part_id=part_id,
            mime_type=mime_type,
            filename=filename,
            headers=headers,
            body=body,
        )

        google_message_payload_parts_item.additional_properties = d
        return google_message_payload_parts_item

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.google_events_events_item_extended_properties_private import (
        GoogleEventsEventsItemExtendedPropertiesPrivate,
    )
    from ..models.google_events_events_item_extended_properties_shared import (
        GoogleEventsEventsItemExtendedPropertiesShared,
    )


T = TypeVar("T", bound="GoogleEventsEventsItemExtendedProperties")


@attr.s(auto_attribs=True)
class GoogleEventsEventsItemExtendedProperties:
    """The extendedProperties of the event

    Attributes:
        private (Union[Unset, GoogleEventsEventsItemExtendedPropertiesPrivate]): The private extendedProperties of the
            event
        shared (Union[Unset, GoogleEventsEventsItemExtendedPropertiesShared]): The shared extendedProperties of the
            event
    """

    private: Union[Unset, "GoogleEventsEventsItemExtendedPropertiesPrivate"] = UNSET
    shared: Union[Unset, "GoogleEventsEventsItemExtendedPropertiesShared"] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        private: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.private, Unset):
            private = self.private.to_dict()

        shared: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.shared, Unset):
            shared = self.shared.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if private is not UNSET:
            field_dict["private"] = private
        if shared is not UNSET:
            field_dict["shared"] = shared

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.google_events_events_item_extended_properties_private import (
            GoogleEventsEventsItemExtendedPropertiesPrivate,
        )
        from ..models.google_events_events_item_extended_properties_shared import (
            GoogleEventsEventsItemExtendedPropertiesShared,
        )

        d = src_dict.copy()
        _private = d.pop("private", UNSET) or UNSET
        private: Union[Unset, GoogleEventsEventsItemExtendedPropertiesPrivate]
        if isinstance(_private, Unset):
            private = UNSET
        else:
            private = GoogleEventsEventsItemExtendedPropertiesPrivate.from_dict(_private)

        _shared = d.pop("shared", UNSET) or UNSET
        shared: Union[Unset, GoogleEventsEventsItemExtendedPropertiesShared]
        if isinstance(_shared, Unset):
            shared = UNSET
        else:
            shared = GoogleEventsEventsItemExtendedPropertiesShared.from_dict(_shared)

        google_events_events_item_extended_properties = cls(
            private=private,
            shared=shared,
        )

        google_events_events_item_extended_properties.additional_properties = d
        return google_events_events_item_extended_properties

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.google_calendar_conference_properties import GoogleCalendarConferenceProperties


T = TypeVar("T", bound="GoogleCalendar")


@attr.s(auto_attribs=True)
class GoogleCalendar:
    """Your google calendar

    Attributes:
        kind (Union[Unset, str]): The kind of the calendar
        etag (Union[Unset, str]): The etag of the calendar
        id (Union[Unset, str]): The id of the calendar
        summary (Union[Unset, str]): The summary of the calendar
        description (Union[Unset, str]): The description of the calendar
        location (Union[Unset, str]): The location of the calendar
        time_zone (Union[Unset, str]): The timeZone of the calendar
        conference_properties (Union[Unset, GoogleCalendarConferenceProperties]): The conferenceProperties of the
            calendar
    """

    kind: Union[Unset, str] = UNSET
    etag: Union[Unset, str] = UNSET
    id: Union[Unset, str] = UNSET
    summary: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    location: Union[Unset, str] = UNSET
    time_zone: Union[Unset, str] = UNSET
    conference_properties: Union[Unset, "GoogleCalendarConferenceProperties"] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        kind = self.kind
        etag = self.etag
        id = self.id
        summary = self.summary
        description = self.description
        location = self.location
        time_zone = self.time_zone
        conference_properties: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.conference_properties, Unset):
            conference_properties = self.conference_properties.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if kind is not UNSET:
            field_dict["kind"] = kind
        if etag is not UNSET:
            field_dict["etag"] = etag
        if id is not UNSET:
            field_dict["id"] = id
        if summary is not UNSET:
            field_dict["summary"] = summary
        if description is not UNSET:
            field_dict["description"] = description
        if location is not UNSET:
            field_dict["location"] = location
        if time_zone is not UNSET:
            field_dict["timeZone"] = time_zone
        if conference_properties is not UNSET:
            field_dict["conferenceProperties"] = conference_properties

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.google_calendar_conference_properties import GoogleCalendarConferenceProperties

        d = src_dict.copy()
        kind = d.pop("kind", UNSET) or UNSET

        etag = d.pop("etag", UNSET) or UNSET

        id = d.pop("id", UNSET) or UNSET

        summary = d.pop("summary", UNSET) or UNSET

        description = d.pop("description", UNSET) or UNSET

        location = d.pop("location", UNSET) or UNSET

        time_zone = d.pop("timeZone", UNSET) or UNSET

        _conference_properties = d.pop("conferenceProperties", UNSET) or UNSET
        conference_properties: Union[Unset, GoogleCalendarConferenceProperties]
        if isinstance(_conference_properties, Unset):
            conference_properties = UNSET
        else:
            conference_properties = GoogleCalendarConferenceProperties.from_dict(_conference_properties)

        google_calendar = cls(
            kind=kind,
            etag=etag,
            id=id,
            summary=summary,
            description=description,
            location=location,
            time_zone=time_zone,
            conference_properties=conference_properties,
        )

        google_calendar.additional_properties = d
        return google_calendar

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
